@@ -1,4 +1,4 @@
-/* $Id: chars.c,v 1.75 2005/06/29 18:17:54 dolorous Exp $ */
+/* $Id: chars.c,v 1.94 2005/09/14 20:20:58 dolorous Exp $ */
 /**************************************************************************
  *   chars.c                                                              *
  *                                                                        *
@@ -24,19 +24,25 @@
 #include <config.h>
 #endif
 
-#include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
-#include <assert.h>
 #include "proto.h"
 
-#ifdef NANO_WIDE
+#ifdef ENABLE_UTF8
 #ifdef HAVE_WCHAR_H
 #include <wchar.h>
 #endif
 #ifdef HAVE_WCTYPE_H
 #include <wctype.h>
 #endif
+
+static const wchar_t bad_wchar = 0xFFFD;
+	/* If we get an invalid multibyte sequence, we treat it as
+	 * Unicode FFFD (Replacement Character), unless we're
+	 * determining if it's a control character or searching for a
+	 * match to it. */
+static const char *bad_mbchar = "\xEF\xBF\xBD";
+static const int bad_mbchar_len = 3;
 #endif
 
 #ifndef HAVE_ISBLANK
@@ -47,7 +53,7 @@ bool nisblank(int c)
 }
 #endif
 
-#if !defined(HAVE_ISWBLANK) && defined(NANO_WIDE)
+#if !defined(HAVE_ISWBLANK) && defined(ENABLE_UTF8)
 /* This function is equivalent to iswblank(). */
 bool niswblank(wchar_t wc)
 {
@@ -67,14 +73,13 @@ bool is_alnum_mbchar(const char *c)
 {
     assert(c != NULL);
 
-#ifdef NANO_WIDE
+#ifdef ENABLE_UTF8
     if (ISSET(USE_UTF8)) {
 	wchar_t wc;
-	int c_mb_len = mbtowc(&wc, c, MB_CUR_MAX);
 
-	if (c_mb_len <= 0) {
+	if (mbtowc(&wc, c, MB_CUR_MAX) < 0) {
 	    mbtowc(NULL, NULL, 0);
-	    wc = (unsigned char)*c;
+	    wc = bad_wchar;
 	}
 
 	return iswalnum(wc);
@@ -88,14 +93,13 @@ bool is_blank_mbchar(const char *c)
 {
     assert(c != NULL);
 
-#ifdef NANO_WIDE
+#ifdef ENABLE_UTF8
     if (ISSET(USE_UTF8)) {
 	wchar_t wc;
-	int c_mb_len = mbtowc(&wc, c, MB_CUR_MAX);
 
-	if (c_mb_len <= 0) {
+	if (mbtowc(&wc, c, MB_CUR_MAX) < 0) {
 	    mbtowc(NULL, NULL, 0);
-	    wc = (unsigned char)*c;
+	    wc = bad_wchar;
 	}
 
 	return iswblank(wc);
@@ -105,14 +109,14 @@ bool is_blank_mbchar(const char *c)
 }
 
 /* This function is equivalent to iscntrl(), except in that it also
- * handles control characters with their high bits set. */
+ * handles high-bit control characters. */
 bool is_cntrl_char(int c)
 {
     return (-128 <= c && c < -96) || (0 <= c && c < 32) ||
 	(127 <= c && c < 160);
 }
 
-#ifdef NANO_WIDE
+#ifdef ENABLE_UTF8
 /* This function is equivalent to iscntrl() for wide characters, except
  * in that it also handles wide control characters with their high bits
  * set. */
@@ -129,14 +133,13 @@ bool is_cntrl_mbchar(const char *c)
 {
     assert(c != NULL);
 
-#ifdef NANO_WIDE
+#ifdef ENABLE_UTF8
     if (ISSET(USE_UTF8)) {
 	wchar_t wc;
-	int c_mb_len = mbtowc(&wc, c, MB_CUR_MAX);
 
-	if (c_mb_len <= 0) {
+	if (mbtowc(&wc, c, MB_CUR_MAX) < 0) {
 	    mbtowc(NULL, NULL, 0);
-	    wc = (unsigned char)*c;
+	    wc = bad_wchar;
 	}
 
 	return is_cntrl_wchar(wc);
@@ -150,14 +153,14 @@ bool is_punct_mbchar(const char *c)
 {
     assert(c != NULL);
 
-#ifdef NANO_WIDE
+#ifdef ENABLE_UTF8
     if (ISSET(USE_UTF8)) {
 	wchar_t wc;
 	int c_mb_len = mbtowc(&wc, c, MB_CUR_MAX);
 
-	if (c_mb_len <= 0) {
+	if (c_mb_len < 0) {
 	    mbtowc(NULL, NULL, 0);
-	    wc = (unsigned char)*c;
+	    wc = bad_wchar;
 	}
 
 	return iswpunct(wc);
@@ -178,9 +181,11 @@ bool is_word_mbchar(const char *c, bool allow_punct)
 }
 
 /* c is a control character.  It displays as ^@, ^?, or ^[ch], where ch
- * is c + 64.  We return that character. */
+ * is (c + 64).  We return that character. */
 char control_rep(char c)
 {
+    assert(is_cntrl_char(c));
+
     /* Treat newlines embedded in a line as encoded nulls. */
     if (c == '\n')
 	return '@';
@@ -190,11 +195,13 @@ char control_rep(char c)
 	return c + 64;
 }
 
-#ifdef NANO_WIDE
+#ifdef ENABLE_UTF8
 /* c is a wide control character.  It displays as ^@, ^?, or ^[ch],
- * where ch is c + 64.  We return that wide character. */
+ * where ch is (c + 64).  We return that wide character. */
 wchar_t control_wrep(wchar_t wc)
 {
+    assert(is_cntrl_wchar(wc));
+
     /* Treat newlines embedded in a line as encoded nulls. */
     if (wc == '\n')
 	return '@';
@@ -206,31 +213,69 @@ wchar_t control_wrep(wchar_t wc)
 #endif
 
 /* c is a multibyte control character.  It displays as ^@, ^?, or ^[ch],
- * where ch is c + 64.  We return that multibyte character. */
+ * where ch is (c + 64).  We return that multibyte character.  If crep
+ * is an invalid multibyte sequence, it will be replaced with Unicode
+ * 0xFFFD (Replacement Character). */
 char *control_mbrep(const char *c, char *crep, int *crep_len)
 {
     assert(c != NULL && crep != NULL && crep_len != NULL);
 
-#ifdef NANO_WIDE
+#ifdef ENABLE_UTF8
     if (ISSET(USE_UTF8)) {
 	wchar_t wc;
 
-	if (mbtowc(&wc, c, MB_CUR_MAX) <= 0) {
+	if (mbtowc(&wc, c, MB_CUR_MAX) < 0) {
 	    mbtowc(NULL, NULL, 0);
-	    wc = (unsigned char)*c;
-	}
+	    *crep_len = bad_mbchar_len;
+	    strncpy(crep, bad_mbchar, *crep_len);
+	} else {
+	    *crep_len = wctomb(crep, control_wrep(wc));
 
-	*crep_len = wctomb(crep, control_wrep(wc));
-
-	if (*crep_len <= 0) {
-	    wctomb(NULL, 0);
-	    *crep_len = 0;
+	    if (*crep_len < 0) {
+		wctomb(NULL, 0);
+		*crep_len = 0;
+	    }
 	}
     } else {
 #endif
 	*crep_len = 1;
 	*crep = control_rep(*c);
-#ifdef NANO_WIDE
+#ifdef ENABLE_UTF8
+    }
+#endif
+
+    return crep;
+}
+
+/* c is a multibyte non-control character.  We return that multibyte
+ * character.  If crep is an invalid multibyte sequence, it will be
+ * replaced with Unicode 0xFFFD (Replacement Character). */
+char *mbrep(const char *c, char *crep, int *crep_len)
+{
+    assert(c != NULL && crep != NULL && crep_len != NULL);
+
+#ifdef ENABLE_UTF8
+    if (ISSET(USE_UTF8)) {
+	wchar_t wc;
+
+	/* Reject invalid Unicode characters. */
+	if (mbtowc(&wc, c, MB_CUR_MAX) < 0 || !is_valid_unicode(wc)) {
+	    mbtowc(NULL, NULL, 0);
+	    *crep_len = bad_mbchar_len;
+	    strncpy(crep, bad_mbchar, *crep_len);
+	} else {
+	    *crep_len = wctomb(crep, wc);
+
+	    if (*crep_len < 0) {
+		wctomb(NULL, 0);
+		*crep_len = 0;
+	    }
+	}
+    } else {
+#endif
+	*crep_len = 1;
+	*crep = *c;
+#ifdef ENABLE_UTF8
     }
 #endif
 
@@ -242,14 +287,14 @@ int mbwidth(const char *c)
 {
     assert(c != NULL);
 
-#ifdef NANO_WIDE
+#ifdef ENABLE_UTF8
     if (ISSET(USE_UTF8)) {
 	wchar_t wc;
-	int c_mb_len = mbtowc(&wc, c, MB_CUR_MAX), width;
+	int width;
 
-	if (c_mb_len <= 0) {
+	if (mbtowc(&wc, c, MB_CUR_MAX) < 0) {
 	    mbtowc(NULL, NULL, 0);
-	    wc = (unsigned char)*c;
+	    wc = bad_wchar;
 	}
 
 	width = wcwidth(wc);
@@ -267,29 +312,30 @@ int mbwidth(const char *c)
 int mb_cur_max(void)
 {
     return
-#ifdef NANO_WIDE
+#ifdef ENABLE_UTF8
 	ISSET(USE_UTF8) ? MB_CUR_MAX :
 #endif
 	1;
 }
 
-/* Convert the value in chr to a multibyte character with the same
- * wide character value as chr, if possible.  If the conversion
+/* Convert the Unicode value in chr to a multibyte character with the
+ * same wide character value as chr, if possible.  If the conversion
  * succeeds, return the (dynamically allocated) multibyte character and
  * its length.  Otherwise, return an undefined (dynamically allocated)
  * multibyte character and a length of zero. */
-char *make_mbchar(int chr, int *chr_mb_len)
+char *make_mbchar(long chr, int *chr_mb_len)
 {
     char *chr_mb;
 
     assert(chr_mb_len != NULL);
 
-#ifdef NANO_WIDE
+#ifdef ENABLE_UTF8
     if (ISSET(USE_UTF8)) {
 	chr_mb = charalloc(MB_CUR_MAX);
-	*chr_mb_len = wctomb(chr_mb, chr);
+	*chr_mb_len = wctomb(chr_mb, (wchar_t)chr);
 
-	if (*chr_mb_len <= 0) {
+	/* Reject invalid Unicode characters. */
+	if (*chr_mb_len < 0 || !is_valid_unicode((wchar_t)chr)) {
 	    wctomb(NULL, 0);
 	    *chr_mb_len = 0;
 	}
@@ -297,7 +343,7 @@ char *make_mbchar(int chr, int *chr_mb_len)
 #endif
 	*chr_mb_len = 1;
 	chr_mb = mallocstrncpy(NULL, (char *)&chr, 1);
-#ifdef NANO_WIDE
+#ifdef ENABLE_UTF8
     }
 #endif
 
@@ -306,33 +352,26 @@ char *make_mbchar(int chr, int *chr_mb_len)
 
 /* Parse a multibyte character from buf.  Return the number of bytes
  * used.  If chr isn't NULL, store the multibyte character in it.  If
- * bad_chr isn't NULL, set it to TRUE if we have a bad multibyte
- * character.  If col isn't NULL, store the new display width in it.  If
- * *str is '\t', we expect col to have the current display width. */
-int parse_mbchar(const char *buf, char *chr, bool *bad_chr, size_t
-	*col)
+ * col isn't NULL, store the new display width in it.  If *buf is '\t',
+ * we expect col to have the current display width. */
+int parse_mbchar(const char *buf, char *chr, size_t *col)
 {
     int buf_mb_len;
 
     assert(buf != NULL);
 
-    if (bad_chr != NULL)
-	*bad_chr = FALSE;
-
-#ifdef NANO_WIDE
+#ifdef ENABLE_UTF8
     if (ISSET(USE_UTF8)) {
 	/* Get the number of bytes in the multibyte character. */
 	buf_mb_len = mblen(buf, MB_CUR_MAX);
 
-	/* If buf contains a null byte or an invalid multibyte
-	 * character, set bad_chr to TRUE (if it contains the latter)
-	 * and interpret buf's first byte. */
-	if (buf_mb_len <= 0) {
+	/* If buf contains an invalid multibyte character, set bad_chr
+	 * to TRUE and interpret buf's first byte. */
+	if (buf_mb_len < 0) {
 	    mblen(NULL, 0);
-	    if (buf_mb_len < 0 && bad_chr != NULL)
-		*bad_chr = TRUE;
 	    buf_mb_len = 1;
-	}
+	} else if (buf_mb_len == 0)
+	    buf_mb_len++;
 
 	/* Save the multibyte character in chr. */
 	if (chr != NULL) {
@@ -393,7 +432,7 @@ int parse_mbchar(const char *buf, char *chr, bool *bad_chr, size_t
 	    else
 		(*col)++;
 	}
-#ifdef NANO_WIDE
+#ifdef ENABLE_UTF8
     }
 #endif
 
@@ -411,8 +450,7 @@ size_t move_mbleft(const char *buf, size_t pos)
     /* There is no library function to move backward one multibyte
      * character.  Here is the naive, O(pos) way to do it. */
     while (TRUE) {
-	int buf_mb_len = parse_mbchar(buf + pos - pos_prev, NULL, NULL,
-		NULL);
+	int buf_mb_len = parse_mbchar(buf + pos - pos_prev, NULL, NULL);
 
 	if (pos_prev <= (size_t)buf_mb_len)
 	    break;
@@ -427,7 +465,7 @@ size_t move_mbleft(const char *buf, size_t pos)
  * after the one at pos. */
 size_t move_mbright(const char *buf, size_t pos)
 {
-    return pos + parse_mbchar(buf + pos, NULL, NULL, NULL);
+    return pos + parse_mbchar(buf + pos, NULL, NULL);
 }
 
 #ifndef HAVE_STRCASECMP
@@ -466,7 +504,7 @@ int nstrncasecmp(const char *s1, const char *s2, size_t n)
  * strings. */
 int mbstrncasecmp(const char *s1, const char *s2, size_t n)
 {
-#ifdef NANO_WIDE
+#ifdef ENABLE_UTF8
     if (ISSET(USE_UTF8)) {
 	char *s1_mb = charalloc(MB_CUR_MAX);
 	char *s2_mb = charalloc(MB_CUR_MAX);
@@ -478,17 +516,17 @@ int mbstrncasecmp(const char *s1, const char *s2, size_t n)
 	    bool bad_s1_mb = FALSE, bad_s2_mb = FALSE;
 	    int s1_mb_len, s2_mb_len;
 
-	    s1_mb_len = parse_mbchar(s1, s1_mb, NULL, NULL);
+	    s1_mb_len = parse_mbchar(s1, s1_mb, NULL);
 
-	    if (mbtowc(&ws1, s1_mb, s1_mb_len) <= 0) {
+	    if (mbtowc(&ws1, s1_mb, s1_mb_len) < 0) {
 		mbtowc(NULL, NULL, 0);
 		ws1 = (unsigned char)*s1_mb;
 		bad_s1_mb = TRUE;
 	    }
 
-	    s2_mb_len = parse_mbchar(s2, s2_mb, NULL, NULL);
+	    s2_mb_len = parse_mbchar(s2, s2_mb, NULL);
 
-	    if (mbtowc(&ws2, s2_mb, s2_mb_len) <= 0) {
+	    if (mbtowc(&ws2, s2_mb, s2_mb_len) < 0) {
 		mbtowc(NULL, NULL, 0);
 		ws2 = (unsigned char)*s2_mb;
 		bad_s2_mb = TRUE;
@@ -536,7 +574,7 @@ const char *nstrcasestr(const char *haystack, const char *needle)
 /* This function is equivalent to strcasestr() for multibyte strings. */
 const char *mbstrcasestr(const char *haystack, const char *needle)
 {
-#ifdef NANO_WIDE
+#ifdef ENABLE_UTF8
     if (ISSET(USE_UTF8)) {
 	char *r_mb = charalloc(MB_CUR_MAX);
 	char *q_mb = charalloc(MB_CUR_MAX);
@@ -552,17 +590,17 @@ const char *mbstrcasestr(const char *haystack, const char *needle)
 	    while (*q != '\0') {
 		bool bad_r_mb = FALSE, bad_q_mb = FALSE;
 
-		r_mb_len = parse_mbchar(r, r_mb, NULL, NULL);
+		r_mb_len = parse_mbchar(r, r_mb, NULL);
 
-		if (mbtowc(&wr, r_mb, r_mb_len) <= 0) {
+		if (mbtowc(&wr, r_mb, r_mb_len) < 0) {
 		    mbtowc(NULL, NULL, 0);
 		    wr = (unsigned char)*r;
 		    bad_r_mb = TRUE;
 		}
 
-		q_mb_len = parse_mbchar(q, q_mb, NULL, NULL);
+		q_mb_len = parse_mbchar(q, q_mb, NULL);
 
-		if (mbtowc(&wq, q_mb, q_mb_len) <= 0) {
+		if (mbtowc(&wq, q_mb, q_mb_len) < 0) {
 		    mbtowc(NULL, NULL, 0);
 		    wq = (unsigned char)*q;
 		    bad_q_mb = TRUE;
@@ -642,7 +680,7 @@ const char *revstrcasestr(const char *haystack, const char *needle,
 const char *mbrevstrcasestr(const char *haystack, const char *needle,
 	const char *rev_start)
 {
-#ifdef NANO_WIDE
+#ifdef ENABLE_UTF8
     if (ISSET(USE_UTF8)) {
 	char *r_mb = charalloc(MB_CUR_MAX);
 	char *q_mb = charalloc(MB_CUR_MAX);
@@ -658,17 +696,17 @@ const char *mbrevstrcasestr(const char *haystack, const char *needle,
 	    while (*q != '\0') {
 		bool bad_r_mb = FALSE, bad_q_mb = FALSE;
 
-		r_mb_len = parse_mbchar(r, r_mb, NULL, NULL);
+		r_mb_len = parse_mbchar(r, r_mb, NULL);
 
-		if (mbtowc(&wr, r_mb, r_mb_len) <= 0) {
+		if (mbtowc(&wr, r_mb, r_mb_len) < 0) {
 		    mbtowc(NULL, NULL, 0);
 		    wr = (unsigned char)*r;
 		    bad_r_mb = TRUE;
 		}
 
-		q_mb_len = parse_mbchar(q, q_mb, NULL, NULL);
+		q_mb_len = parse_mbchar(q, q_mb, NULL);
 
-		if (mbtowc(&wq, q_mb, q_mb_len) <= 0) {
+		if (mbtowc(&wq, q_mb, q_mb_len) < 0) {
 		    mbtowc(NULL, NULL, 0);
 		    wq = (unsigned char)*q;
 		    bad_q_mb = TRUE;
@@ -730,13 +768,13 @@ size_t mbstrnlen(const char *s, size_t maxlen)
 {
     assert(s != NULL);
 
-#ifdef NANO_WIDE
+#ifdef ENABLE_UTF8
     if (ISSET(USE_UTF8)) {
 	size_t n = 0;
 	int s_mb_len;
 
 	while (*s != '\0') {
-	    s_mb_len = parse_mbchar(s, NULL, NULL, NULL);
+	    s_mb_len = parse_mbchar(s, NULL, NULL);
 
 	    if (maxlen == 0)
 		break;
@@ -753,6 +791,52 @@ size_t mbstrnlen(const char *s, size_t maxlen)
 }
 
 #ifndef DISABLE_JUSTIFY
+/* This function is equivalent to strchr() for multibyte strings. */
+char *mbstrchr(const char *s, char *c)
+{
+    assert(s != NULL && c != NULL);
+
+#ifdef ENABLE_UTF8
+    if (ISSET(USE_UTF8)) {
+	bool bad_c_mb = FALSE, bad_s_mb = FALSE;
+	char *s_mb = charalloc(MB_CUR_MAX);
+	const char *q = s;
+	wchar_t ws, wc;
+	int c_mb_len = mbtowc(&wc, c, MB_CUR_MAX);
+
+	if (c_mb_len < 0) {
+	    mbtowc(NULL, NULL, 0);
+	    wc = (unsigned char)*c;
+	    bad_c_mb = TRUE;
+	}
+
+	while (*s != '\0') {
+	    int s_mb_len = parse_mbchar(s, s_mb, NULL);
+
+	    if (mbtowc(&ws, s_mb, s_mb_len) < 0) {
+		mbtowc(NULL, NULL, 0);
+		ws = (unsigned char)*s;
+		bad_s_mb = TRUE;
+	    }
+
+	    if (bad_s_mb == bad_c_mb && ws == wc)
+		break;
+
+	    s += s_mb_len;
+	    q += s_mb_len;
+	}
+
+	free(s_mb);
+
+	if (ws != wc)
+	    q = NULL;
+
+	return (char *)q;
+    } else
+#endif
+	return strchr(s, *c);
+}
+
 #ifdef ENABLE_NANORC
 /* Return TRUE if the string s contains one or more blank characters,
  * and FALSE otherwise. */
@@ -774,7 +858,7 @@ bool has_blank_mbchars(const char *s)
 {
     assert(s != NULL);
 
-#ifdef NANO_WIDE
+#ifdef ENABLE_UTF8
     if (ISSET(USE_UTF8)) {
 	char *chr_mb = charalloc(MB_CUR_MAX);
 	bool retval = FALSE;
@@ -782,7 +866,7 @@ bool has_blank_mbchars(const char *s)
 	while (*s != '\0') {
 	    int chr_mb_len;
 
-	    chr_mb_len = parse_mbchar(s, chr_mb, NULL, NULL);
+	    chr_mb_len = parse_mbchar(s, chr_mb, NULL);
 
 	    if (is_blank_mbchar(chr_mb)) {
 		retval = TRUE;
@@ -800,51 +884,18 @@ bool has_blank_mbchars(const char *s)
 	return has_blank_chars(s);
 }
 #endif /* ENABLE_NANORC */
-
-/* This function is equivalent to strchr() for multibyte strings. */
-char *mbstrchr(const char *s, char *c)
-{
-    assert(s != NULL && c != NULL);
-
-#ifdef NANO_WIDE
-    if (ISSET(USE_UTF8)) {
-	char *s_mb = charalloc(MB_CUR_MAX);
-	const char *q = s;
-	wchar_t ws, wc;
-	int c_mb_len = mbtowc(&wc, c, MB_CUR_MAX);
-
-	if (c_mb_len <= 0) {
-	    mbtowc(NULL, NULL, 0);
-	    wc = (unsigned char)*c;
-	}
-
-	while (*s != '\0') {
-	    int s_mb_len = parse_mbchar(s, s_mb, NULL, NULL);
-
-	    if (mbtowc(&ws, s_mb, s_mb_len) <= 0) {
-		mbtowc(NULL, NULL, 0);
-		ws = (unsigned char)*s;
-	    }
-
-	    if (ws == wc)
-		break;
-
-	    s += s_mb_len;
-	    q += s_mb_len;
-	}
-
-	free(s_mb);
-
-	if (ws != wc)
-	    q = NULL;
-
-	return (char *)q;
-    } else
-#endif
-	return strchr(s, *c);
-}
 #endif /* !DISABLE_JUSTIFY */
 
+#ifdef ENABLE_UTF8
+/* Return TRUE if wc is valid Unicode, and FALSE otherwise. */
+bool is_valid_unicode(wchar_t wc)
+{
+    return ((0 <= wc && wc <= 0x10FFFF) && (wc <= 0xD7FF || 0xE000 <=
+	wc) && (wc <= 0xFDCF || 0xFDF0 <= wc) && ((wc & 0xFFFF) <=
+	0xFFFD));
+}
+#endif
+
 #ifdef ENABLE_NANORC
 /* Check if the string s is a valid multibyte string.  Return TRUE if it
  * is, and FALSE otherwise. */
@@ -853,7 +904,7 @@ bool is_valid_mbstring(const char *s)
     assert(s != NULL);
 
     return 
-#ifdef NANO_WIDE
+#ifdef ENABLE_UTF8
 	ISSET(USE_UTF8) ?
 	(mbstowcs(NULL, s, 0) != (size_t)-1) :
 #endif
