@@ -1,4 +1,4 @@
-/* $Id: color.c,v 1.13 2005/06/19 21:10:48 dolorous Exp $ */
+/* $Id: color.c,v 1.33 2005/08/01 05:54:11 dolorous Exp $ */
 /**************************************************************************
  *   color.c                                                              *
  *                                                                        *
@@ -24,11 +24,8 @@
 #include <config.h>
 #endif
 
-#include <stdlib.h>
-#include <string.h>
 #include <stdio.h>
-#include <errno.h>
-#include <fcntl.h>
+#include <string.h>
 #include "proto.h"
 
 #ifdef ENABLE_COLOR
@@ -63,70 +60,147 @@ void set_colorpairs(void)
     }
 }
 
-void do_colorinit(void)
+/* Initialize the color information. */
+void color_init(void)
 {
+    assert(openfile != NULL);
+
     if (has_colors()) {
-	const colortype *tmpcolor = NULL;
+	const colortype *tmpcolor;
 #ifdef HAVE_USE_DEFAULT_COLORS
 	bool defok;
 #endif
 
 	start_color();
 
-	/* Add in colors, if available. */
 #ifdef HAVE_USE_DEFAULT_COLORS
+	/* Add in colors, if available. */
 	defok = (use_default_colors() != ERR);
 #endif
 
-	for (tmpcolor = colorstrings; tmpcolor != NULL;
+	for (tmpcolor = openfile->colorstrings; tmpcolor != NULL;
 		tmpcolor = tmpcolor->next) {
-	    short background = tmpcolor->bg;
+	    short foreground = tmpcolor->fg, background = tmpcolor->bg;
+	    if (foreground == -1) {
+#ifdef HAVE_USE_DEFAULT_COLORS
+		if (!defok)
+#endif
+		    foreground = COLOR_WHITE;
+	    }
 
-	    if (background == -1)
+	    if (background == -1) {
 #ifdef HAVE_USE_DEFAULT_COLORS
 		if (!defok)
 #endif
 		    background = COLOR_BLACK;
+	    }
 
-	    init_pair(tmpcolor->pairnum, tmpcolor->fg, background);
+	    init_pair(tmpcolor->pairnum, foreground, background);
 
 #ifdef DEBUG
-	    fprintf(stderr, "init_pair(): fg = %d, bg = %d\n",
-		tmpcolor->fg, tmpcolor->bg);
+	    fprintf(stderr, "init_pair(): fg = %hd, bg = %hd\n", tmpcolor->fg, tmpcolor->bg);
 #endif
 	}
     }
 }
 
 /* Update the color information based on the current filename. */
-void update_color(void)
+void color_update(void)
 {
     const syntaxtype *tmpsyntax;
+    colortype *tmpcolor, *defcolor = NULL;
 
-    colorstrings = NULL;
-    for (tmpsyntax = syntaxes; tmpsyntax != NULL;
-	tmpsyntax = tmpsyntax->next) {
-	const exttype *e;
+    assert(openfile != NULL);
 
-	for (e = tmpsyntax->extensions; e != NULL; e = e->next) {
-	    /* Set colorstrings if we matched the extension regex. */
-	    if (regexec(&e->val, filename, 0, NULL, 0) == 0)
-		colorstrings = tmpsyntax->color;
+    openfile->colorstrings = NULL;
 
-	    if (colorstrings != NULL)
+    /* If we specified a syntax override string, use it. */
+    if (syntaxstr != NULL) {
+	/* If the syntax override is "none", it's the same as not having
+	 * a syntax at all, so get out. */
+	if (strcmp(syntaxstr, "none") == 0)
+	    return;
+
+	for (tmpsyntax = syntaxes; tmpsyntax != NULL;
+		tmpsyntax = tmpsyntax->next) {
+	    if (strcmp(tmpsyntax->desc, syntaxstr) == 0)
+		openfile->colorstrings = tmpsyntax->color;
+
+	    if (openfile->colorstrings != NULL)
 		break;
 	}
     }
 
-    /* If we haven't found a match, use the override string. */
-    if (colorstrings == NULL && syntaxstr != NULL) {
+    /* If we didn't specify a syntax override string, or if we did and
+     * there was no syntax by that name, get the syntax based on the
+     * file extension. */
+    if (openfile->colorstrings == NULL) {
 	for (tmpsyntax = syntaxes; tmpsyntax != NULL;
 		tmpsyntax = tmpsyntax->next) {
-	    if (mbstrcasecmp(tmpsyntax->desc, syntaxstr) == 0)
-		colorstrings = tmpsyntax->color;
+	    exttype *e;
+
+	    /* If this is the default syntax, it has no associated
+	     * extensions.  (We've checked for this and for duplicate
+	     * syntax names elsewhere.)  Skip over it here, but keep
+	     * track of its color regexes. */
+	    if (strcmp(tmpsyntax->desc, "default") == 0) {
+		defcolor = syntaxes->color;
+		continue;
+	    }
+
+	    for (e = tmpsyntax->extensions; e != NULL; e = e->next) {
+		bool not_compiled = (e->ext == NULL);
+
+		/* e->ext_regex has already been checked for validity
+		 * elsewhere.  Compile its specified regex if we haven't
+		 * already. */
+		if (not_compiled) {
+		    e->ext = (regex_t *)nmalloc(sizeof(regex_t));
+		    regcomp(e->ext, e->ext_regex, REG_EXTENDED);
+		}
+
+		/* Set colorstrings if we matched the extension
+		 * regex. */
+		if (regexec(e->ext, openfile->filename, 0, NULL,
+			0) == 0)
+		    openfile->colorstrings = tmpsyntax->color;
+
+		if (openfile->colorstrings != NULL)
+		    break;
+
+		/* Decompile e->ext_regex's specified regex if we aren't
+		 * going to use it. */
+		if (not_compiled) {
+		    regfree(e->ext);
+		    free(e->ext);
+		    e->ext = NULL;
+		}
+	    }
+	}
+    }
+
+    /* If we didn't get a syntax based on the file extension, and we
+     * have a default syntax, use it. */
+    if (openfile->colorstrings == NULL && defcolor != NULL)
+	openfile->colorstrings = defcolor;
+
+    for (tmpcolor = openfile->colorstrings; tmpcolor != NULL;
+	tmpcolor = tmpcolor->next) {
+	/* tmpcolor->start_regex and tmpcolor->end_regex have already
+	 * been checked for validity elsewhere.  Compile their specified
+	 * regexes if we haven't already. */
+	if (tmpcolor->start == NULL) {
+	    tmpcolor->start = (regex_t *)nmalloc(sizeof(regex_t));
+	    regcomp(tmpcolor->start, tmpcolor->start_regex,
+		REG_EXTENDED | (tmpcolor->icase ? REG_ICASE : 0));
+	}
+
+	if (tmpcolor->end_regex != NULL && tmpcolor->end == NULL) {
+	    tmpcolor->end = (regex_t *)nmalloc(sizeof(regex_t));
+	    regcomp(tmpcolor->end, tmpcolor->end_regex,
+		REG_EXTENDED | (tmpcolor->icase ? REG_ICASE : 0));
 	}
     }
-    do_colorinit();
 }
 
 #endif /* ENABLE_COLOR */
