@@ -1,4 +1,4 @@
-/* $Id: global.c,v 1.126 2005/06/28 06:25:34 dolorous Exp $ */
+/* $Id: global.c,v 1.144 2005/09/13 04:53:44 dolorous Exp $ */
 /**************************************************************************
  *   global.c                                                             *
  *                                                                        *
@@ -24,43 +24,29 @@
 #include <config.h>
 #endif
 
-#include <stdlib.h>
-#include <assert.h>
 #include "proto.h"
 
 /* Global variables */
-
 #ifndef DISABLE_WRAPJUSTIFY
-/* wrap_at might be set in rcfile.c or nano.c. */
-ssize_t wrap_at = -CHARS_FROM_EOL;	/* Right justified fill value,
-					   allows resize */
+ssize_t fill = 0;		/* The column where we will wrap
+				 * lines. */
+ssize_t wrap_at = -CHARS_FROM_EOL;
+				/* The position where we will wrap
+				 * lines.  fill is equal to this if it's
+				 * greater than zero, and equal to
+				 * (COLS + this) if it isn't. */
 #endif
 
 char *last_search = NULL;	/* Last string we searched for */
 char *last_replace = NULL;	/* Last replacement string */
 
-unsigned long flags = 0;	/* Our flag containing many options */
-WINDOW *topwin;			/* Top buffer */
+long flags = 0;			/* Our flag containing many options */
+WINDOW *topwin;			/* Top subwindow */
 WINDOW *edit;			/* The file portion of the editor */
-WINDOW *bottomwin;		/* Bottom buffer */
-char *filename = NULL;		/* Name of the file */
-
-#ifndef NANO_SMALL
-struct stat originalfilestat;	/* Stat for the file as we loaded it */
-#endif
+WINDOW *bottomwin;		/* Bottom subwindow */
 
 int editwinrows = 0;		/* How many rows long is the edit
 				   window? */
-filestruct *current;		/* Current buffer pointer */
-size_t current_x = 0;		/* Current x-coordinate in the edit
-				   window */
-ssize_t current_y = 0;		/* Current y-coordinate in the edit
-				   window */
-filestruct *fileage = NULL;	/* Our file buffer */
-filestruct *edittop = NULL;	/* Pointer to the top of the edit
-				   buffer with respect to the
-				   file struct */
-filestruct *filebot = NULL;	/* Last node in the file struct */
 filestruct *cutbuffer = NULL;	/* A place to store cut text */
 #ifndef DISABLE_JUSTIFY
 filestruct *jusbuffer = NULL;	/* A place to store unjustified text */
@@ -68,10 +54,8 @@ filestruct *jusbuffer = NULL;	/* A place to store unjustified text */
 partition *filepart = NULL;	/* A place to store a portion of the
 				   file struct */
 
-#ifdef ENABLE_MULTIBUFFER
-openfilestruct *open_files = NULL;	/* The list of open file
-					   buffers */
-#endif
+openfilestruct *openfile = NULL;
+				/* The list of open file buffers */
 
 #if !defined(NANO_SMALL) && defined(ENABLE_NANORC)
 char *whitespace = NULL;	/* Characters used when displaying
@@ -102,28 +86,14 @@ char *backup_dir = NULL;	/* Backup directory. */
 #endif
 
 char *answer = NULL;		/* Answer str to many questions */
-size_t totlines = 0;		/* Total number of lines in the file */
-size_t totsize = 0;		/* Total number of characters in the
-				   file */
-size_t placewewant = 0;		/* The column we'd like the cursor
-				   to jump to when we go to the
-				   next or previous line */
 
 ssize_t tabsize = -1;		/* Our internal tabsize variable.  The
 				   default value is set in main(). */
 
-char *hblank = NULL;		/* A horizontal blank line */
 #ifndef DISABLE_HELP
 char *help_text;		/* The text in the help window */
 #endif
 
-/* More stuff for the marker select */
-
-#ifndef NANO_SMALL
-filestruct *mark_beginbuf;	/* The begin marker buffer */
-size_t mark_beginx;		/* X value in the string to start */
-#endif
-
 #ifndef DISABLE_OPERATINGDIR
 char *operating_dir = NULL;	/* Operating directory, which we can't */
 char *full_operating_dir = NULL;/* go higher than */
@@ -155,7 +125,6 @@ shortcut *gotodir_list = NULL;
 #endif
 
 #ifdef ENABLE_COLOR
-const colortype *colorstrings = NULL;
 syntaxtype *syntaxes = NULL;
 char *syntaxstr = NULL;
 #endif
@@ -197,7 +166,8 @@ size_t length_of_list(const shortcut *s)
     return i;
 }
 
-/* Initialize a struct *without* our lovely braces =( */
+/* Create a new shortcut structure, at the end of the shortcuts linked
+ * list. */
 void sc_init_one(shortcut **shortcutage, int ctrlval, const char *desc,
 #ifndef DISABLE_HELP
 	const char *help,
@@ -292,8 +262,7 @@ void shortcut_init(bool unjustify)
 	N_("Show the position of the cursor");
     const char *nano_spell_msg =
 	N_("Invoke the spell checker, if available");
-    const char *nano_gotoline_msg =
-	N_("Go to a specific line number and column number");
+    const char *nano_gotoline_msg = N_("Go to line and column number");
     const char *nano_replace_msg = N_("Replace text within the editor");
 #ifndef NANO_SMALL
     const char *nano_mark_msg = N_("Mark text at the cursor position");
@@ -321,7 +290,7 @@ void shortcut_init(bool unjustify)
     const char *nano_nextword_msg = N_("Move forward one word");
     const char *nano_prevword_msg = N_("Move backward one word");
     const char *nano_wordcount_msg =
-	N_("Count the number of words in the file");
+	N_("Count the number of words, lines, and characters");
 #endif
 #ifndef DISABLE_JUSTIFY
     const char *nano_parabegin_msg =
@@ -330,9 +299,9 @@ void shortcut_init(bool unjustify)
 	N_("Go to the end of the current paragraph");
 #endif
 #ifdef ENABLE_MULTIBUFFER
-    const char *nano_openprev_msg =
+    const char *nano_prevfile_msg =
 	N_("Switch to the previous file buffer");
-    const char *nano_opennext_msg =
+    const char *nano_nextfile_msg =
 	N_("Switch to the next file buffer");
 #endif
     const char *nano_verbatim_msg = N_("Insert character(s) verbatim");
@@ -412,7 +381,7 @@ void shortcut_init(bool unjustify)
     /* Translators: try to keep this string under 10 characters long */
     sc_init_one(&main_list, NANO_EXIT_KEY,
 #ifdef ENABLE_MULTIBUFFER
-	open_files != NULL && open_files != open_files->next ?
+	openfile != NULL && openfile != openfile->next ?
 	N_("Close") :
 #endif
 	exit_msg, IFHELP(nano_exit_msg, NANO_NO_KEY), NANO_EXIT_FKEY,
@@ -528,11 +497,11 @@ void shortcut_init(bool unjustify)
 
     sc_init_one(&main_list, NANO_FORWARD_KEY, N_("Forward"),
 	IFHELP(nano_forward_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, do_right_void);
+	NANO_NO_KEY, VIEW, do_right);
 
     sc_init_one(&main_list, NANO_BACK_KEY, N_("Back"),
 	IFHELP(nano_back_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, do_left_void);
+	NANO_NO_KEY, VIEW, do_left);
 
     sc_init_one(&main_list, NANO_HOME_KEY, N_("Home"),
 	IFHELP(nano_home_msg, NANO_NO_KEY), NANO_NO_KEY,
@@ -573,7 +542,7 @@ void shortcut_init(bool unjustify)
 
     sc_init_one(&main_list, NANO_NO_KEY, N_("Word Count"),
 	IFHELP(nano_wordcount_msg, NANO_WORDCOUNT_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, do_word_count);
+	NANO_NO_KEY, VIEW, do_wordlinechar_count);
 #endif
 
 #ifndef DISABLE_JUSTIFY
@@ -590,12 +559,12 @@ void shortcut_init(bool unjustify)
 
 #ifdef ENABLE_MULTIBUFFER
     sc_init_one(&main_list, NANO_NO_KEY, N_("Previous File"),
-	IFHELP(nano_openprev_msg, NANO_OPENPREV_KEY), NANO_NO_KEY,
-	NANO_OPENPREV_ALTKEY, VIEW, open_prevfile_void);
+	IFHELP(nano_prevfile_msg, NANO_PREVFILE_KEY), NANO_NO_KEY,
+	NANO_PREVFILE_ALTKEY, VIEW, switch_to_prev_buffer_void);
 
     sc_init_one(&main_list, NANO_NO_KEY, N_("Next File"),
-	IFHELP(nano_opennext_msg, NANO_OPENNEXT_KEY), NANO_NO_KEY,
-	NANO_OPENNEXT_ALTKEY, VIEW, open_nextfile_void);
+	IFHELP(nano_nextfile_msg, NANO_NEXTFILE_KEY), NANO_NO_KEY,
+	NANO_NEXTFILE_ALTKEY, VIEW, switch_to_next_buffer_void);
 #endif
 
     sc_init_one(&main_list, NANO_NO_KEY, N_("Verbatim Input"),
@@ -1074,7 +1043,7 @@ void free_shortcutage(shortcut **shortcutage)
 }
 
 #ifndef NANO_SMALL
-/* Create one new toggle structure, at the end of the toggles linked
+/* Create a new toggle structure, at the end of the toggles linked
  * list. */
 void toggle_init_one(int val, const char *desc, long flag)
 {
@@ -1191,8 +1160,6 @@ void thanks_for_all_the_fish(void)
 	free(last_search);
     if (last_replace != NULL)
 	free(last_replace);
-    if (hblank != NULL)
-	free(hblank);
 #ifndef DISABLE_SPELLER
     if (alt_speller != NULL)
 	free(alt_speller);
@@ -1201,8 +1168,6 @@ void thanks_for_all_the_fish(void)
     if (help_text != NULL)
 	free(help_text);
 #endif
-    if (filename != NULL)
-	free(filename);
     if (answer != NULL)
 	free(answer);
     if (cutbuffer != NULL)
@@ -1240,19 +1205,9 @@ void thanks_for_all_the_fish(void)
 	free(t);
     }
 #endif
-#ifdef ENABLE_MULTIBUFFER
     /* Free the memory associated with each open file buffer. */
-    if (open_files != NULL) {
-	/* Make sure open_files->fileage is up to date, in case we've
-	 * cut the top line of the file. */
-	open_files->fileage = fileage;
-
-	free_openfilestruct(open_files);
-    }
-#else
-    if (fileage != NULL)
-	free_filestruct(fileage);
-#endif
+    if (openfile != NULL)
+	free_openfilestruct(openfile);
 #ifdef ENABLE_COLOR
     if (syntaxstr != NULL)
 	free(syntaxstr);
@@ -1264,17 +1219,28 @@ void thanks_for_all_the_fish(void)
 	    exttype *bob = syntaxes->extensions;
 
 	    syntaxes->extensions = bob->next;
-	    regfree(&bob->val);
+	    free(bob->ext_regex);
+	    if (bob->ext != NULL) {
+		regfree(bob->ext);
+		free(bob->ext);
+	    }
 	    free(bob);
 	}
 	while (syntaxes->color != NULL) {
 	    colortype *bob = syntaxes->color;
 
 	    syntaxes->color = bob->next;
-	    regfree(&bob->start);
-	    if (bob->end != NULL)
+	    free(bob->start_regex);
+	    if (bob->start != NULL) {
+		regfree(bob->start);
+		free(bob->start);
+	    }
+	    if (bob->end_regex != NULL)
+		free(bob->end_regex);
+	    if (bob->end != NULL) {
 		regfree(bob->end);
-	    free(bob->end);
+		free(bob->end);
+	    }
 	    free(bob);
 	}
 	syntaxes = syntaxes->next;
