@@ -1,4 +1,4 @@
-/* $Id: nano.h,v 1.117 2005/06/28 06:25:34 dolorous Exp $ */
+/* $Id: nano.h,v 1.137 2005/09/26 02:58:16 dolorous Exp $ */
 /**************************************************************************
  *   nano.h                                                               *
  *                                                                        *
@@ -90,19 +90,20 @@
 	/* Mark a string that will be sent to gettext() later. */
 
 #include <stddef.h>
+#include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <dirent.h>
+#ifdef HAVE_REGEX_H
+#include <regex.h>
+#endif
+#include <assert.h>
 
-/* If no snprintf() or vsnprintf(), use the versions from glib 2.x. */
-#if !defined(HAVE_SNPRINTF) || !defined(HAVE_VSNPRINTF)
-#include <glib.h>
-#ifndef HAVE_SNPRINTF
-#define snprintf g_snprintf
-#endif
+/* If no vsnprintf(), use the version from glib 2.x. */
 #ifndef HAVE_VSNPRINTF
+#include <glib.h>
 #define vsnprintf g_vsnprintf
 #endif
-#endif
 
 /* If no isblank(), iswblank(), strcasecmp(), strncasecmp(),
  * strcasestr(), strnlen(), getdelim(), or getline(), use the versions
@@ -134,6 +135,12 @@
 
 #define VERMSG "GNU nano " VERSION
 
+/* If we don't have regex support, turn the color support off, as it
+ * depends on the use of regexes. */
+#ifndef HAVE_REGEX_H
+#undef ENABLE_COLOR 
+#endif
+
 /* If we aren't using ncurses, turn the mouse support off, as it's
  * ncurses-specific. */
 #ifndef NCURSES_MOUSE_VERSION
@@ -150,12 +157,16 @@ typedef enum {
 } file_format;
 
 typedef enum {
-    UP, DOWN
-} updown;
+    OVERWRITE, APPEND, PREPEND
+} append_type;
 
 typedef enum {
-    TOP, CENTER, NONE
-} topmidnone;
+    UP, DOWN
+} scroll_dir;
+
+typedef enum {
+    CENTER, NONE
+} update_type;
 
 /* Structure types. */
 typedef struct filestruct {
@@ -165,40 +176,6 @@ typedef struct filestruct {
     ssize_t lineno;		/* The line number. */
 } filestruct;
 
-#ifdef ENABLE_MULTIBUFFER
-typedef struct openfilestruct {
-    char *filename;
-#ifndef NANO_SMALL
-    struct stat originalfilestat;
-#endif
-    struct openfilestruct *next;
-				/* Next node. */
-    struct openfilestruct *prev;
-				/* Previous node. */
-    filestruct *fileage;	/* Current file. */
-    filestruct *filebot;	/* Current file's last line. */
-    filestruct *edittop;	/* Current top of edit window. */
-    filestruct *current;	/* Current file's line. */
-#ifndef NANO_SMALL
-    filestruct *mark_beginbuf;
-				/* Current file's beginning marked
-				 * line. */
-    size_t mark_beginx;		/* Current file's beginning marked
-				 * line's x-coordinate position. */
-#endif
-    size_t current_x;		/* Current file's x-coordinate
-				 * position. */
-    size_t placewewant;		/* Current file's place we want. */
-    size_t totlines;		/* Current file's total number of
-				 * lines. */
-    size_t totsize;		/* Current file's total size. */
-    unsigned long flags;	/* Current file's flags: modification
-				 * status (and marking status, if
-				 * available). */
-    file_format fmt;		/* Current file's format. */
-} openfilestruct;
-#endif
-
 typedef struct partition {
     filestruct *fileage;
     filestruct *top_prev;
@@ -208,6 +185,74 @@ typedef struct partition {
     char *bot_data;
 } partition;
 
+#ifdef ENABLE_COLOR
+typedef struct colortype {
+    short fg;			/* Foreground color. */
+    short bg;			/* Background color. */
+    bool bright;		/* Is this color A_BOLD? */
+    bool icase;			/* Is this regex string case
+				 * insensitive? */
+    int pairnum;		/* Color pair number used for this
+				 * foreground/background. */
+    char *start_regex;		/* Start (or all) of the regex
+				 * string. */
+    regex_t *start;		/* Compiled start (or all) of the regex
+				 * string. */
+    char *end_regex;		/* End (if any) of the regex string. */
+    regex_t *end;		/* Compiled end (if any) of the regex
+				 * string. */
+    struct colortype *next;
+} colortype;
+
+typedef struct exttype {
+    char *ext_regex;		/* Extensions that match this syntax. */
+    regex_t *ext;		/* Compiled extensions that match this
+				 * syntax. */
+    struct exttype *next;
+} exttype;
+
+typedef struct syntaxtype {
+    char *desc;			/* Name of this syntax type. */
+    exttype *extensions;	/* List of extensions that this syntax
+				 * applies to. */
+    colortype *color;		/* Color struct for this syntax. */
+    struct syntaxtype *next;
+} syntaxtype;
+#endif /* ENABLE_COLOR */
+
+typedef struct openfilestruct {
+    char *filename;		/* Current file's name. */
+    filestruct *fileage;	/* Current file's first line. */
+    filestruct *filebot;	/* Current file's last line. */
+    filestruct *edittop;	/* Current top of edit window. */
+    filestruct *current;	/* Current file's line. */
+    size_t totsize;		/* Current file's total number of
+				 * characters. */
+    size_t current_x;		/* Current file's x-coordinate
+				 * position. */
+    size_t placewewant;		/* Current file's place we want. */
+    ssize_t current_y;		/* Current file's y-coordinate
+				 * position. */
+    bool modified;		/* Current file's modification
+				 * status. */
+#ifndef NANO_SMALL
+    bool mark_set;		/* Current file's marking status. */
+    filestruct *mark_begin;	/* Current file's beginning marked
+				 * line. */
+    size_t mark_begin_x;	/* Current file's beginning marked
+				 * line's x-coordinate position. */
+    file_format fmt;		/* Current file's format. */
+    struct stat *current_stat;	/* Current file's stat. */
+#endif
+#ifdef ENABLE_COLOR
+    colortype *colorstrings;	/* Current file's associated colors. */
+#endif
+    struct openfilestruct *next;
+				/* Next node. */
+    struct openfilestruct *prev;
+				/* Previous node. */
+} openfilestruct;
+
 typedef struct shortcut {
     /* Key values that aren't used should be set to NANO_NO_KEY. */
     int ctrlval;	/* Special sentinel key or control key we want
@@ -243,66 +288,38 @@ typedef struct rcoption {
 } rcoption;
 #endif
 
-#ifdef ENABLE_COLOR
-typedef struct colortype {
-    int fg;			/* Foreground color. */
-    int bg;			/* Background color. */
-    bool bright;		/* Is this color A_BOLD? */
-    int pairnum;		/* Color pair number used for this
-				 * foreground/background. */
-    regex_t start;		/* Start (or all) of the regex
-				 * string. */
-    regex_t *end;		/* End (if any) of the regex string. */
-    struct colortype *next;
-} colortype;
-
-typedef struct exttype {
-    regex_t val;		/* The extensions that match this
-				 * syntax. */
-    struct exttype *next;
-} exttype;
-
-typedef struct syntaxtype {
-    char *desc;			/* Name of this syntax type. */
-    exttype *extensions;	/* List of extensions that this syntax
-				 * applies to. */
-    colortype *color;		/* Color struct for this syntax. */
-    struct syntaxtype *next;
-} syntaxtype;
-#endif
-
 /* Bitwise flags so that we can save space (or, more correctly, not
  * waste it). */
-#define MODIFIED		(1<<0)
-#define CASE_SENSITIVE		(1<<1)
-#define MARK_ISSET		(1<<2)
-#define CONST_UPDATE		(1<<3)
-#define NO_HELP			(1<<4)
-#define NOFOLLOW_SYMLINKS	(1<<5)
-#define SUSPEND			(1<<6)
-#define NO_WRAP			(1<<7)
-#define AUTOINDENT		(1<<8)
-#define VIEW_MODE		(1<<9)
-#define USE_MOUSE		(1<<10)
-#define USE_REGEXP		(1<<11)
-#define TEMP_FILE		(1<<12)
-#define CUT_TO_END		(1<<13)
-#define BACKWARDS_SEARCH	(1<<14)
-#define MULTIBUFFER		(1<<15)
-#define SMOOTH_SCROLL		(1<<16)
-#define REBIND_DELETE		(1<<17)
-#define NO_CONVERT		(1<<18)
-#define BACKUP_FILE		(1<<19)
-#define NO_RCFILE		(1<<20)
-#define NO_COLOR_SYNTAX		(1<<21)
-#define PRESERVE		(1<<22)
-#define HISTORYLOG		(1<<23)
-#define RESTRICTED		(1<<24)
-#define SMART_HOME		(1<<25)
-#define WHITESPACE_DISPLAY	(1<<26)
-#define MORE_SPACE		(1<<27)
-#define TABS_TO_SPACES		(1<<28)
-#define QUICK_BLANK		(1<<29)
+#define CASE_SENSITIVE		(1<<0)
+#define CONST_UPDATE		(1<<1)
+#define NO_HELP			(1<<2)
+#define NOFOLLOW_SYMLINKS	(1<<3)
+#define SUSPEND			(1<<4)
+#define NO_WRAP			(1<<5)
+#define AUTOINDENT		(1<<6)
+#define VIEW_MODE		(1<<7)
+#define USE_MOUSE		(1<<8)
+#define USE_REGEXP		(1<<9)
+#define TEMP_FILE		(1<<10)
+#define CUT_TO_END		(1<<11)
+#define BACKWARDS_SEARCH	(1<<12)
+#define MULTIBUFFER		(1<<13)
+#define SMOOTH_SCROLL		(1<<14)
+#define REBIND_DELETE		(1<<15)
+#define REBIND_KEYPAD		(1<<16)
+#define NO_CONVERT		(1<<17)
+#define BACKUP_FILE		(1<<18)
+#define NO_RCFILE		(1<<19)
+#define NO_COLOR_SYNTAX		(1<<20)
+#define PRESERVE		(1<<21)
+#define HISTORYLOG		(1<<22)
+#define RESTRICTED		(1<<23)
+#define SMART_HOME		(1<<24)
+#define WHITESPACE_DISPLAY	(1<<25)
+#define MORE_SPACE		(1<<26)
+#define TABS_TO_SPACES		(1<<27)
+#define QUICK_BLANK		(1<<28)
+#define WORD_BOUNDS		(1<<29)
 #define USE_UTF8		(1<<30)
 
 /* Control key sequences.  Changing these would be very, very bad. */
@@ -340,8 +357,16 @@ typedef struct syntaxtype {
 #define NANO_CONTROL_7 31
 #define NANO_CONTROL_8 127
 
+#define NANO_ALT_SPACE ' '
+#define NANO_ALT_LPAREN '('
+#define NANO_ALT_RPAREN ')'
+#define NANO_ALT_COMMA ','
+#define NANO_ALT_PERIOD '.'
 #define NANO_ALT_9 '9'
 #define NANO_ALT_0 '0'
+#define NANO_ALT_LCARAT '<'
+#define NANO_ALT_RCARAT '>'
+#define NANO_ALT_RBRACKET ']'
 #define NANO_ALT_A 'a'
 #define NANO_ALT_B 'b'
 #define NANO_ALT_C 'c'
@@ -368,14 +393,6 @@ typedef struct syntaxtype {
 #define NANO_ALT_X 'x'
 #define NANO_ALT_Y 'y'
 #define NANO_ALT_Z 'z'
-#define NANO_ALT_PERIOD '.'
-#define NANO_ALT_COMMA ','
-#define NANO_ALT_LPAREN '('
-#define NANO_ALT_RPAREN ')'
-#define NANO_ALT_LCARAT '<'
-#define NANO_ALT_RCARAT '>'
-#define NANO_ALT_RBRACKET ']'
-#define NANO_ALT_SPACE ' '
 
 /* Some semi-changeable keybindings; don't play with these unless you're
  * sure you know what you're doing.  Assume ERR is defined as -1. */
@@ -448,10 +465,10 @@ typedef struct syntaxtype {
 #define NANO_TOFILES_KEY	NANO_CONTROL_T
 #define NANO_APPEND_KEY		NANO_ALT_A
 #define NANO_PREPEND_KEY	NANO_ALT_P
-#define NANO_OPENPREV_KEY	NANO_ALT_LCARAT
-#define NANO_OPENNEXT_KEY	NANO_ALT_RCARAT
-#define NANO_OPENPREV_ALTKEY	NANO_ALT_COMMA
-#define NANO_OPENNEXT_ALTKEY	NANO_ALT_PERIOD
+#define NANO_PREVFILE_KEY	NANO_ALT_LCARAT
+#define NANO_NEXTFILE_KEY	NANO_ALT_RCARAT
+#define NANO_PREVFILE_ALTKEY	NANO_ALT_COMMA
+#define NANO_NEXTFILE_ALTKEY	NANO_ALT_PERIOD
 #define NANO_BRACKET_KEY	NANO_ALT_RBRACKET
 #define NANO_NEXTWORD_KEY	NANO_CONTROL_SPACE
 #define NANO_PREVWORD_KEY	NANO_ALT_SPACE
