@@ -1,4 +1,4 @@
-/* $Id: proto.h,v 1.238 2005/06/30 03:55:55 dolorous Exp $ */
+/* $Id: proto.h,v 1.280 2005/09/13 04:53:44 dolorous Exp $ */
 /**************************************************************************
  *   proto.h                                                              *
  *                                                                        *
@@ -24,28 +24,14 @@
 #define PROTO_H 1
 
 /* Externs. */
-
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <dirent.h>
-#ifdef HAVE_REGEX_H
-#include <regex.h>
-#endif
 #include "nano.h"
 
 #ifndef DISABLE_WRAPJUSTIFY
+extern ssize_t fill;
 extern ssize_t wrap_at;
 #endif
 extern int editwinrows;
-extern size_t current_x;
-extern ssize_t current_y;
-extern size_t totlines;
-extern size_t placewewant;
-#ifndef NANO_SMALL
-extern size_t mark_beginx;
-#endif
-extern size_t totsize;
-extern unsigned long flags;
+extern long flags;
 extern ssize_t tabsize;
 extern int currslen;
 
@@ -72,10 +58,7 @@ extern char *backup_dir;
 #endif
 
 extern WINDOW *topwin, *edit, *bottomwin;
-extern char *filename;
-extern struct stat originalfilestat;
 extern char *answer;
-extern char *hblank;
 #ifndef DISABLE_HELP
 extern char *help_text;
 #endif
@@ -90,22 +73,15 @@ extern char *alt_speller;
 #endif
 
 extern struct stat fileinfo;
-extern filestruct *current, *fileage, *edittop, *filebot;
 extern filestruct *cutbuffer;
 #ifndef DISABLE_JUSTIFY
 extern filestruct *jusbuffer;
 #endif
 extern partition *filepart;
-#ifndef NANO_SMALL
-extern filestruct *mark_beginbuf;
-#endif
 
-#ifdef ENABLE_MULTIBUFFER
-extern openfilestruct *open_files;
-#endif
+extern openfilestruct *openfile;
 
 #ifdef ENABLE_COLOR
-extern const colortype *colorstrings;
 extern syntaxtype *syntaxes;
 extern char *syntaxstr;
 #endif
@@ -162,29 +138,29 @@ extern char *homedir;
 #ifndef HAVE_ISBLANK
 bool nisblank(int c);
 #endif
-#if !defined(HAVE_ISWBLANK) && defined(NANO_WIDE)
+#if !defined(HAVE_ISWBLANK) && defined(ENABLE_UTF8)
 bool niswblank(wchar_t wc);
 #endif
 bool is_byte(int c);
 bool is_alnum_mbchar(const char *c);
 bool is_blank_mbchar(const char *c);
 bool is_cntrl_char(int c);
-#ifdef NANO_WIDE
+#ifdef ENABLE_UTF8
 bool is_cntrl_wchar(wchar_t wc);
 #endif
 bool is_cntrl_mbchar(const char *c);
 bool is_punct_mbchar(const char *c);
 bool is_word_mbchar(const char *c, bool allow_punct);
 char control_rep(char c);
-#ifdef NANO_WIDE
+#ifdef ENABLE_UTF8
 wchar_t control_wrep(wchar_t c);
 #endif
 char *control_mbrep(const char *c, char *crep, int *crep_len);
+char *mbrep(const char *c, char *crep, int *crep_len);
 int mbwidth(const char *c);
 int mb_cur_max(void);
-char *make_mbchar(int chr, int *chr_mb_len);
-int parse_mbchar(const char *buf, char *chr, bool *bad_chr, size_t
-	*col);
+char *make_mbchar(long chr, int *chr_mb_len);
+int parse_mbchar(const char *buf, char *chr, size_t *col);
 size_t move_mbleft(const char *buf, size_t pos);
 size_t move_mbright(const char *buf, size_t pos);
 #ifndef HAVE_STRCASECMP
@@ -215,11 +191,14 @@ size_t nstrnlen(const char *s, size_t maxlen);
 #endif
 size_t mbstrnlen(const char *s, size_t maxlen);
 #ifndef DISABLE_JUSTIFY
+char *mbstrchr(const char *s, char *c);
 #ifdef ENABLE_NANORC
 bool has_blank_chars(const char *s);
 bool has_blank_mbchars(const char *s);
 #endif
-char *mbstrchr(const char *s, char *c);
+#endif
+#ifdef ENABLE_UTF8
+bool is_valid_unicode(wchar_t wc);
 #endif
 #ifdef ENABLE_NANORC
 bool is_valid_mbstring(const char *s);
@@ -228,8 +207,8 @@ bool is_valid_mbstring(const char *s);
 /* Public functions in color.c. */
 #ifdef ENABLE_COLOR
 void set_colorpairs(void);
-void do_colorinit(void);
-void update_color(void);
+void color_init(void);
+void color_update(void);
 #endif /* ENABLE_COLOR */
 
 /* Public functions in cut.c. */
@@ -246,17 +225,22 @@ void do_cut_till_end(void);
 void do_uncut_text(void);
 
 /* Public functions in files.c. */
-void new_file(void);
+void make_new_buffer(void);
+void initialize_buffer(void);
+void initialize_buffer_text(void);
+void open_buffer(const char *filename);
+void display_buffer(void);
+#ifdef ENABLE_MULTIBUFFER
+void switch_to_prevnext_buffer(bool next);
+void switch_to_prev_buffer_void(void);
+void switch_to_next_buffer_void(void);
+bool close_buffer(void);
+#endif
 filestruct *read_line(char *buf, filestruct *prevnode, bool
 	*first_line_ins, size_t buf_len);
-void load_file(void);
 void read_file(FILE *f, const char *filename);
 int open_file(const char *filename, bool newfie, FILE **f);
 char *get_next_filename(const char *name, const char *suffix);
-#ifndef NANO_SMALL
-void execute_command(const char *command);
-#endif
-void load_buffer(const char *name);
 void do_insertfile(
 #ifndef NANO_SMALL
 	bool execute
@@ -265,22 +249,6 @@ void do_insertfile(
 #endif
 	);
 void do_insertfile_void(void);
-#ifdef ENABLE_MULTIBUFFER
-openfilestruct *make_new_opennode(void);
-void splice_opennode(openfilestruct *begin, openfilestruct *newnode,
-	openfilestruct *end);
-void unlink_opennode(openfilestruct *fileptr);
-void delete_opennode(openfilestruct *fileptr);
-#ifdef DEBUG
-void free_openfilestruct(openfilestruct *src);
-#endif
-void add_open_file(bool update);
-void load_open_file(void);
-void open_prevnext_file(bool next_file);
-void open_prevfile_void(void);
-void open_nextfile_void(void);
-bool close_open_file(void);
-#endif
 char *get_full_path(const char *origpath);
 char *check_writable_directory(const char *path);
 char *safe_tempfile(FILE **f);
@@ -292,10 +260,11 @@ bool check_operating_dir(const char *currpath, bool allow_tabcomp);
 void init_backup_dir(void);
 #endif
 int copy_file(FILE *inn, FILE *out);
-int write_file(const char *name, FILE *f_open, bool tmp, int append,
-	bool nonamechange);
+int write_file(const char *name, FILE *f_open, bool tmp, append_type
+	append, bool nonamechange);
 #ifndef NANO_SMALL
-int write_marked(const char *name, FILE *f_open, bool tmp, int append);
+int write_marked_file(const char *name, FILE *f_open, bool tmp,
+	append_type append);
 #endif
 int do_writeout(bool exiting);
 void do_writeout_void(void);
@@ -347,32 +316,28 @@ void thanks_for_all_the_fish(void);
 /* Public functions in move.c. */
 void do_first_line(void);
 void do_last_line(void);
-void do_home(void);
-void do_end(void);
 void do_page_up(void);
 void do_page_down(void);
+#ifndef DISABLE_JUSTIFY
+void do_para_begin(bool allow_update);
+void do_para_begin_void(void);
+void do_para_end(bool allow_update);
+void do_para_end_void(void);
+#endif
+#ifndef NANO_SMALL
+bool do_next_word(bool allow_punct, bool allow_update);
+void do_next_word_void(void);
+bool do_prev_word(bool allow_punct, bool allow_update);
+void do_prev_word_void(void);
+#endif
+void do_home(void);
+void do_end(void);
 void do_up(void);
 void do_down(void);
-void do_left(bool allow_update);
-void do_left_void(void);
-void do_right(bool allow_update);
-void do_right_void(void);
+void do_left(void);
+void do_right(void);
 
 /* Public functions in nano.c. */
-void print_view_warning(void);
-void finish(void);
-void die(const char *msg, ...);
-void die_save_file(const char *die_filename);
-void check_die_too_small(void);
-void resize_variables(void);
-void global_init(bool save_cutbuffer);
-void window_init(void);
-#ifndef DISABLE_MOUSE
-void mouse_init(void);
-#endif
-#ifndef DISABLE_HELP
-void help_init(void);
-#endif
 filestruct *make_new_node(filestruct *prevnode);
 filestruct *copy_node(const filestruct *src);
 void splice_node(filestruct *begin, filestruct *newnode, filestruct
@@ -381,14 +346,32 @@ void unlink_node(const filestruct *fileptr);
 void delete_node(filestruct *fileptr);
 filestruct *copy_filestruct(const filestruct *src);
 void free_filestruct(filestruct *src);
+void renumber(filestruct *fileptr);
 partition *partition_filestruct(filestruct *top, size_t top_x,
 	filestruct *bot, size_t bot_x);
 void unpartition_filestruct(partition **p);
 void move_to_filestruct(filestruct **file_top, filestruct **file_bot,
 	filestruct *top, size_t top_x, filestruct *bot, size_t bot_x);
 void copy_from_filestruct(filestruct *file_top, filestruct *file_bot);
-void renumber_all(void);
-void renumber(filestruct *fileptr);
+openfilestruct *make_new_opennode(void);
+void splice_opennode(openfilestruct *begin, openfilestruct *newnode,
+	openfilestruct *end);
+void unlink_opennode(openfilestruct *fileptr);
+void delete_opennode(openfilestruct *fileptr);
+#ifdef DEBUG
+void free_openfilestruct(openfilestruct *src);
+#endif
+void print_view_warning(void);
+void finish(void);
+void die(const char *msg, ...);
+void die_save_file(const char *die_filename);
+void window_init(void);
+#ifndef DISABLE_MOUSE
+void mouse_init(void);
+#endif
+#ifndef DISABLE_HELP
+void help_init(void);
+#endif
 void print1opt_full(const char *shortflag
 #ifdef HAVE_GETOPT_LONG
 	, const char *longflag
@@ -399,59 +382,7 @@ void version(void);
 int no_more_space(void);
 int no_help(void);
 void nano_disabled_msg(void);
-#ifndef NANO_SMALL
-void cancel_fork(int signal);
-bool open_pipe(const char *command);
-#endif
 void do_verbatim_input(void);
-void do_backspace(void);
-void do_delete(void);
-void do_tab(void);
-void do_enter(void);
-#ifndef NANO_SMALL
-bool do_next_word(bool allow_punct, bool allow_update);
-void do_next_word_void(void);
-bool do_prev_word(bool allow_punct, bool allow_update);
-void do_prev_word_void(void);
-void do_word_count(void);
-void do_mark(void);
-#endif
-#ifndef DISABLE_WRAPPING
-void wrap_reset(void);
-bool do_wrap(filestruct *line);
-#endif
-#ifndef DISABLE_SPELLER
-bool do_int_spell_fix(const char *word);
-const char *do_int_speller(const char *tempfile_name);
-const char *do_alt_speller(char *tempfile_name);
-void do_spell(void);
-#endif
-#if !defined(DISABLE_HELP) || !defined(DISABLE_JUSTIFY) || !defined(DISABLE_WRAPPING)
-ssize_t break_line(const char *line, ssize_t goal, bool newline);
-#endif
-#if !defined(NANO_SMALL) || !defined(DISABLE_JUSTIFY)
-size_t indent_length(const char *line);
-#endif
-#ifndef DISABLE_JUSTIFY
-void justify_format(filestruct *paragraph, size_t skip);
-size_t quote_length(const char *line);
-bool quotes_match(const char *a_line, size_t a_quote, const char
-	*b_line);
-bool indents_match(const char *a_line, size_t a_indent, const char
-	*b_line, size_t b_indent);
-bool begpar(const filestruct *const foo);
-void do_para_begin(bool allow_update);
-void do_para_begin_void(void);
-bool inpar(const filestruct *const foo);
-void do_para_end(bool allow_update);
-void do_para_end_void(void);
-filestruct *backup_lines(filestruct *first_line, size_t par_len, size_t
-	quote_len);
-bool find_paragraph(size_t *const quote, size_t *const par);
-void do_justify(bool full_justify);
-void do_justify_void(void);
-void do_full_justify(void);
-#endif /* !DISABLE_JUSTIFY */
 void do_exit(void);
 void signal_init(void);
 void handle_hupterm(int signal);
@@ -485,9 +416,9 @@ void rcfile_error(const char *msg, ...);
 char *parse_next_word(char *ptr);
 char *parse_argument(char *ptr);
 #ifdef ENABLE_COLOR
-int color_to_int(const char *colorname, bool *bright);
+short color_to_short(const char *colorname, bool *bright);
 char *parse_next_regex(char *ptr);
-bool nregcomp(regex_t *preg, const char *regex, int eflags);
+bool nregcomp(const char *regex, int eflags);
 void parse_syntax(char *ptr);
 void parse_colors(char *ptr, bool icase);
 #endif /* ENABLE_COLOR */
@@ -504,9 +435,8 @@ void not_found_msg(const char *str);
 void search_abort(void);
 void search_init_globals(void);
 int search_init(bool replacing, bool use_answer);
-bool is_whole_word(size_t pos, const char *buf, const char *word);
 bool findnextstr(bool can_display_wrap, bool wholeword, bool
-	no_sameline, const filestruct *begin, size_t beginx, const char
+	no_sameline, const filestruct *begin, size_t begin_x, const char
 	*needle, size_t *needle_len);
 void findnextstr_wrap_reset(void);
 void do_search(void);
@@ -523,9 +453,9 @@ ssize_t do_replace_loop(const char *needle, const filestruct
 	*canceled);
 void do_replace(void);
 void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
-	bool interactive, bool save_pos);
+	bool interactive, bool save_pos, bool allow_update);
 void do_gotolinecolumn_void(void);
-#if defined(ENABLE_MULTIBUFFER) || !defined(DISABLE_SPELLER)
+#ifndef DISABLE_SPELLER
 void do_gotopos(ssize_t line, size_t pos_x, ssize_t pos_y, size_t
 	pos_pww);
 #endif
@@ -537,24 +467,66 @@ void do_find_bracket(void);
 bool history_has_changed(void);
 #endif
 void history_init(void);
-filestruct *find_history(filestruct *h_start, filestruct *h_end, const
-	char *s, size_t len);
+void history_reset(const filestruct *h);
+filestruct *find_history(const filestruct *h_start, const filestruct
+	*h_end, const char *s, size_t len);
 void update_history(filestruct **h, const char *s);
 char *get_history_older(filestruct **h);
 char *get_history_newer(filestruct **h);
 #ifndef DISABLE_TABCOMP
-char *get_history_completion(filestruct **h, char *s, size_t len);
+char *get_history_completion(filestruct **h, const char *s, size_t len);
 #endif
 #endif /* !NANO_SMALL */
 
+/* Public functions in text.c. */
+#ifndef NANO_SMALL
+void do_mark(void);
+#endif
+void do_delete(void);
+void do_backspace(void);
+void do_tab(void);
+void do_enter(void);
+#ifndef NANO_SMALL
+void cancel_command(int signal);
+bool execute_command(const char *command);
+#endif
+#ifndef DISABLE_WRAPPING
+void wrap_reset(void);
+bool do_wrap(filestruct *line);
+#endif
+#if !defined(DISABLE_HELP) || !defined(DISABLE_JUSTIFY) || !defined(DISABLE_WRAPPING)
+ssize_t break_line(const char *line, ssize_t goal, bool newline);
+#endif
+#if !defined(NANO_SMALL) || !defined(DISABLE_JUSTIFY)
+size_t indent_length(const char *line);
+#endif
+#ifndef DISABLE_JUSTIFY
+void justify_format(filestruct *paragraph, size_t skip);
+size_t quote_length(const char *line);
+bool quotes_match(const char *a_line, size_t a_quote, const char
+	*b_line);
+bool indents_match(const char *a_line, size_t a_indent, const char
+	*b_line, size_t b_indent);
+bool begpar(const filestruct *const foo);
+bool inpar(const filestruct *const foo);
+filestruct *backup_lines(filestruct *first_line, size_t par_len, size_t
+	quote_len);
+bool find_paragraph(size_t *const quote, size_t *const par);
+void do_justify(bool full_justify);
+void do_justify_void(void);
+void do_full_justify(void);
+#endif /* !DISABLE_JUSTIFY */
+#ifndef DISABLE_SPELLER
+bool do_int_spell_fix(const char *word);
+const char *do_int_speller(const char *tempfile_name);
+const char *do_alt_speller(char *tempfile_name);
+void do_spell(void);
+#endif
+#ifndef NANO_SMALL
+void do_wordlinechar_count(void);
+#endif
+
 /* Public functions in utils.c. */
-#ifdef HAVE_REGEX_H
-#ifdef BROKEN_REGEXEC
-int safe_regexec(const regex_t *preg, const char *string, size_t nmatch,
-	regmatch_t pmatch[], int eflags);
-#endif
-int regexp_bol_or_eol(const regex_t *preg, const char *string);
-#endif
 int digits(size_t n);
 void get_homedir(void);
 bool parse_num(const char *str, ssize_t *val);
@@ -571,6 +543,14 @@ ssize_t ngetline(char **lineptr, size_t *n, FILE *stream);
 ssize_t ngetdelim(char **lineptr, size_t *n, int delim, FILE *stream);
 #endif
 #endif /* !NANO_SMALL && ENABLE_NANORC */
+#ifdef HAVE_REGEX_H
+#ifdef BROKEN_REGEXEC
+int safe_regexec(const regex_t *preg, const char *string, size_t nmatch,
+	regmatch_t pmatch[], int eflags);
+#endif
+int regexp_bol_or_eol(const regex_t *preg, const char *string);
+#endif
+bool is_whole_word(size_t pos, const char *buf, const char *word);
 const char *strstrwrapper(const char *haystack, const char *needle,
 	const char *start);
 void nperror(const char *s);
@@ -585,15 +565,14 @@ void remove_magicline(void);
 void mark_order(const filestruct **top, size_t *top_x, const filestruct
 	**bot, size_t *bot_x, bool *right_side_up);
 #endif
-void get_totals(const filestruct *begin, const filestruct *end, size_t
-	*lines, size_t *size);
+size_t get_totsize(const filestruct *begin, const filestruct *end);
 
 /* Public functions in winio.c. */
 #ifndef NANO_SMALL
 void reset_kbinput(void);
 #endif
-void get_buffer(WINDOW *win);
-size_t get_buffer_len(void);
+void get_key_buffer(WINDOW *win);
+size_t get_key_buffer_len(void);
 void unget_input(int *input, size_t input_len);
 void unget_kbinput(int kbinput, bool meta_key, bool func_key);
 int *get_input(WINDOW *win, size_t input_len);
@@ -611,7 +590,7 @@ int get_byte_kbinput(int kbinput
 	, bool reset
 #endif
 	);
-int get_word_kbinput(int kbinput
+long get_unicode_kbinput(int kbinput
 #ifndef NANO_SMALL
 	, bool reset
 #endif
@@ -633,6 +612,8 @@ int do_statusbar_input(bool *meta_key, bool *func_key, bool *s_or_t,
 #ifndef DISABLE_MOUSE
 bool do_statusbar_mouse(void);
 #endif
+void do_statusbar_output(char *output, size_t output_len, bool
+	*got_enter, bool allow_cntrls);
 void do_statusbar_home(void);
 void do_statusbar_end(void);
 void do_statusbar_right(void);
@@ -645,12 +626,11 @@ bool do_statusbar_next_word(bool allow_punct);
 bool do_statusbar_prev_word(bool allow_punct);
 #endif
 void do_statusbar_verbatim_input(bool *got_enter);
-void do_statusbar_output(char *output, size_t output_len, bool
-	*got_enter, bool allow_cntrls);
 size_t xplustabs(void);
 size_t actual_x(const char *str, size_t xplus);
 size_t strnlenpt(const char *buf, size_t size);
 size_t strlenpt(const char *buf);
+void blank_line(WINDOW *win, int y, int x, int n);
 void blank_titlebar(void);
 void blank_topbar(void);
 void blank_edit(void);
@@ -685,12 +665,12 @@ void reset_cursor(void);
 void edit_add(const filestruct *fileptr, const char *converted, int
 	yval, size_t start);
 void update_line(const filestruct *fileptr, size_t index);
-int need_horizontal_update(size_t old_pww);
-int need_vertical_update(size_t old_pww);
-void edit_scroll(updown direction, int nlines);
+bool need_horizontal_update(size_t old_pww);
+bool need_vertical_update(size_t old_pww);
+void edit_scroll(scroll_dir direction, ssize_t nlines);
 void edit_redraw(const filestruct *old_current, size_t old_pww);
 void edit_refresh(void);
-void edit_update(topmidnone location);
+void edit_update(update_type location);
 int do_yesno(bool all, const char *msg);
 void total_redraw(void);
 void total_refresh(void);
@@ -706,8 +686,8 @@ void do_replace_highlight(bool highlight, const char *word);
 int check_linenumbers(const filestruct *fileptr);
 #endif
 #ifdef DEBUG
-void dump_buffer(const filestruct *inptr);
-void dump_buffer_reverse(void);
+void dump_filestruct(const filestruct *inptr);
+void dump_filestruct_reverse(void);
 #endif
 #ifdef NANO_EXTRA
 void do_credits(void);
