@@ -1,4 +1,4 @@
-/* $Id: rcfile.c,v 1.69 2005/06/28 06:25:34 dolorous Exp $ */
+/* $Id: rcfile.c,v 1.83 2005/08/10 22:12:28 dolorous Exp $ */
 /**************************************************************************
  *   rcfile.c                                                             *
  *                                                                        *
@@ -24,14 +24,12 @@
 #include <config.h>
 #endif
 
-#include <stdlib.h>
 #include <stdarg.h>
 #include <string.h>
 #include <stdio.h>
 #include <errno.h>
 #include <unistd.h>
 #include <ctype.h>
-#include <assert.h>
 #include "proto.h"
 
 #ifdef ENABLE_NANORC
@@ -65,6 +63,7 @@ const static rcoption rcopts[] = {
     {"quotestr", 0},
 #endif
     {"rebinddelete", REBIND_DELETE},
+    {"rebindkeypad", REBIND_KEYPAD},
 #ifdef HAVE_REGEX_H
     {"regexp", USE_REGEXP},
 #endif
@@ -89,6 +88,7 @@ const static rcoption rcopts[] = {
     {"smooth", SMOOTH_SCROLL},
     {"tabstospaces", TABS_TO_SPACES},
     {"whitespace", 0},
+    {"wordbounds", WORD_BOUNDS},
 #endif
     {NULL, 0}
 };
@@ -112,8 +112,7 @@ void rcfile_error(const char *msg, ...)
     fprintf(stderr, "\n");
     if (lineno > 0) {
 	errors = TRUE;
-	fprintf(stderr, _("Error in %s on line %lu: "), nanorc,
-		(unsigned long)lineno);
+	fprintf(stderr, _("Error in %s on line %lu: "), nanorc, (unsigned long)lineno);
     }
 
     va_start(ap, msg);
@@ -182,9 +181,9 @@ char *parse_argument(char *ptr)
 }
 
 #ifdef ENABLE_COLOR
-int color_to_int(const char *colorname, bool *bright)
+short color_to_short(const char *colorname, bool *bright)
 {
-    int mcolor = -1;
+    short mcolor = -1;
 
     assert(colorname != NULL && bright != NULL);
 
@@ -246,27 +245,31 @@ char *parse_next_regex(char *ptr)
     return ptr;
 }
 
-/* Compile the regular expression regex to preg.  Return FALSE on
- * success, or TRUE if the expression is invalid. */
-bool nregcomp(regex_t *preg, const char *regex, int eflags)
+/* Compile the regular expression regex to see if it's valid.  Return
+ * TRUE if it is, or FALSE otherwise. */
+bool nregcomp(const char *regex, int eflags)
 {
-    int rc = regcomp(preg, regex, REG_EXTENDED | eflags);
+    regex_t preg;
+    int rc = regcomp(&preg, regex, REG_EXTENDED | eflags);
 
     if (rc != 0) {
-	size_t len = regerror(rc, preg, NULL, 0);
+	size_t len = regerror(rc, &preg, NULL, 0);
 	char *str = charalloc(len);
 
-	regerror(rc, preg, str, len);
+	regerror(rc, &preg, str, len);
 	rcfile_error(N_("Bad regex \"%s\": %s"), regex, str);
 	free(str);
     }
 
-    return (rc != 0);
+    regfree(&preg);
+
+    return (rc == 0);
 }
 
 void parse_syntax(char *ptr)
 {
     const char *fileregptr = NULL, *nameptr = NULL;
+    const syntaxtype *tmpsyntax;
     exttype *endext = NULL;
 	/* The end of the extensions list for this syntax. */
 
@@ -291,6 +294,14 @@ void parse_syntax(char *ptr)
     if (ptr == NULL)
 	return;
 
+    for (tmpsyntax = syntaxes; tmpsyntax != NULL;
+	tmpsyntax = tmpsyntax->next) {
+	if (strcmp(nameptr, tmpsyntax->desc) == 0) {
+	    rcfile_error(N_("Duplicate syntax name %s"), nameptr);
+	    return;
+	}
+    }
+
     if (syntaxes == NULL) {
 	syntaxes = (syntaxtype *)nmalloc(sizeof(syntaxtype));
 	endsyntax = syntaxes;
@@ -301,15 +312,31 @@ void parse_syntax(char *ptr)
 	fprintf(stderr, "Adding new syntax after first one\n");
 #endif
     }
+
     endsyntax->desc = mallocstrcpy(NULL, nameptr);
     endsyntax->color = NULL;
     endcolor = NULL;
     endsyntax->extensions = NULL;
     endsyntax->next = NULL;
+
 #ifdef DEBUG
     fprintf(stderr, "Starting a new syntax type: \"%s\"\n", nameptr);
 #endif
 
+    /* The "none" syntax is the same as not having a syntax at all, so
+     * we can't assign any extensions or colors to it. */
+    if (strcmp(endsyntax->desc, "none") == 0) {
+	rcfile_error(N_("The \"none\" syntax is reserved"));
+	return;
+    }
+
+    /* The default syntax should have no associated extensions. */
+    if (strcmp(endsyntax->desc, "default") == 0 && *ptr != '\0') {
+	rcfile_error(
+		N_("The \"default\" syntax must take no extensions"));
+	return;
+    }
+
     /* Now load the extensions into their part of the struct. */
     while (*ptr != '\0') {
 	exttype *newext;
@@ -329,16 +356,20 @@ void parse_syntax(char *ptr)
 	    break;
 
 	newext = (exttype *)nmalloc(sizeof(exttype));
-	if (nregcomp(&newext->val, fileregptr, REG_NOSUB))
-	    free(newext);
-	else {
+
+	/* Save the extension regex if it's valid. */
+	if (nregcomp(fileregptr, REG_NOSUB)) {
+	    newext->ext_regex = mallocstrcpy(NULL, fileregptr);
+	    newext->ext = NULL;
+
 	    if (endext == NULL)
 		endsyntax->extensions = newext;
 	    else
 		endext->next = newext;
 	    endext = newext;
 	    endext->next = NULL;
-	}
+	} else
+	    free(newext);
     }
 }
 
@@ -346,7 +377,7 @@ void parse_syntax(char *ptr)
  * treat the color stuff as case insensitive. */
 void parse_colors(char *ptr, bool icase)
 {
-    int fg, bg;
+    short fg, bg;
     bool bright = FALSE, no_fgcolor = FALSE;
     char *fgstr;
 
@@ -377,12 +408,12 @@ void parse_colors(char *ptr, bool icase)
 		bgcolorname);
 	    return;
 	}
-	bg = color_to_int(bgcolorname, &bright);
+	bg = color_to_short(bgcolorname, &bright);
     } else
 	bg = -1;
 
     if (!no_fgcolor) {
-	fg = color_to_int(fgstr, &bright);
+	fg = color_to_short(fgstr, &bright);
 
 	/* Don't try to parse screwed-up foreground colors. */
 	if (fg == -1)
@@ -425,34 +456,45 @@ void parse_colors(char *ptr, bool icase)
 
 	ptr++;
 
-	newcolor = (colortype *)nmalloc(sizeof(colortype));
 	fgstr = ptr;
 	ptr = parse_next_regex(ptr);
 	if (ptr == NULL)
 	    break;
 
-	if (nregcomp(&newcolor->start, fgstr, icase ? REG_ICASE : 0)) {
-	    free(newcolor);
-	    cancelled = TRUE;
-	} else {
+	newcolor = (colortype *)nmalloc(sizeof(colortype));
+
+	/* Save the starting regex string if it's valid, and set up the
+	 * color information. */
+	if (nregcomp(fgstr, icase ? REG_ICASE : 0)) {
 	    newcolor->fg = fg;
 	    newcolor->bg = bg;
 	    newcolor->bright = bright;
-	    newcolor->next = NULL;
+	    newcolor->icase = icase;
+
+	    newcolor->start_regex = mallocstrcpy(NULL, fgstr);
+	    newcolor->start = NULL;
+
+	    newcolor->end_regex = NULL;
 	    newcolor->end = NULL;
 
+	    newcolor->next = NULL;
+
 	    if (endcolor == NULL) {
 		endsyntax->color = newcolor;
 #ifdef DEBUG
-		fprintf(stderr, "Starting a new colorstring for fg %d, bg %d\n", fg, bg);
+		fprintf(stderr, "Starting a new colorstring for fg %hd, bg %hd\n", fg, bg);
 #endif
 	    } else {
 #ifdef DEBUG
-		fprintf(stderr, "Adding new entry for fg %d, bg %d\n", fg, bg);
+		fprintf(stderr, "Adding new entry for fg %hd, bg %hd\n", fg, bg);
 #endif
 		endcolor->next = newcolor;
 	    }
+
 	    endcolor = newcolor;
+	} else {
+	    free(newcolor);
+	    cancelled = TRUE;
 	}
 
 	if (expectend) {
@@ -480,11 +522,9 @@ void parse_colors(char *ptr, bool icase)
 	    if (cancelled)
 		continue;
 
-	    newcolor->end = (regex_t *)nmalloc(sizeof(regex_t));
-	    if (nregcomp(newcolor->end, fgstr, icase ? REG_ICASE : 0)) {
-		free(newcolor->end);
-		newcolor->end = NULL;
-	    }
+	    /* Save the ending regex string if it's valid. */
+	    newcolor->end_regex = (nregcomp(fgstr, icase ? REG_ICASE :
+		0)) ? mallocstrcpy(NULL, fgstr) : NULL;
 	}
     }
 }
@@ -610,11 +650,10 @@ void parse_rcfile(FILE *rcstream)
 			    } else {
 				whitespace_len[0] =
 					parse_mbchar(whitespace, NULL,
-					NULL, NULL);
+					NULL);
 				whitespace_len[1] =
 					parse_mbchar(whitespace +
-					whitespace_len[0], NULL,
-					NULL, NULL);
+					whitespace_len[0], NULL, NULL);
 			    }
 			} else
 #endif
@@ -686,8 +725,7 @@ void parse_rcfile(FILE *rcstream)
 
     if (errors) {
 	errors = FALSE;
-	fprintf(stderr,
-		_("\nPress Return to continue starting nano\n"));
+	fprintf(stderr, _("\nPress Return to continue starting nano\n"));
 	while (getchar() != '\n')
 	    ;
     }
