@@ -1,4 +1,4 @@
-/* $Id: search.c,v 1.140 2005/06/29 00:17:18 dolorous Exp $ */
+/* $Id: search.c,v 1.161 2005/08/11 18:30:47 dolorous Exp $ */
 /**************************************************************************
  *   search.c                                                             *
  *                                                                        *
@@ -24,13 +24,11 @@
 #include <config.h>
 #endif
 
-#include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <ctype.h>
 #include <errno.h>
-#include <assert.h>
 #include "proto.h"
 
 static bool search_last_line = FALSE;
@@ -57,6 +55,7 @@ int regexp_init(const char *regexp)
 	);
 
     assert(!regexp_compiled);
+
     if (rc != 0) {
 	size_t len = regerror(rc, &search_regexp, NULL, 0);
 	char *str = charalloc(len);
@@ -100,7 +99,7 @@ void search_abort(void)
 {
     display_main_list();
 #ifndef NANO_SMALL
-    if (ISSET(MARK_ISSET))
+    if (openfile->mark_set)
 	edit_refresh();
 #endif
 #ifdef HAVE_REGEX_H
@@ -153,7 +152,7 @@ int search_init(bool replacing, bool use_answer)
 	char *disp = display_string(last_search, 0, COLS / 3, FALSE);
 
 	buf = charalloc(strlen(disp) + 7);
-	/* We use COLS / 3 here because we need to see more on the
+	/* We use (COLS / 3) here because we need to see more on the
 	 * line. */
 	sprintf(buf, " [%s%s]", disp,
 		strlenpt(last_search) > COLS / 3 ? "..." : "");
@@ -192,13 +191,9 @@ int search_init(bool replacing, bool use_answer)
 
 	replacing ?
 #ifndef NANO_SMALL
-		(ISSET(MARK_ISSET) ? _(" (to replace) in selection") :
+		openfile->mark_set ? _(" (to replace) in selection") :
 #endif
-		_(" (to replace)")
-#ifndef NANO_SMALL
-		)
-#endif
-		: "",
+		_(" (to replace)") : "",
 
 	buf);
 
@@ -215,17 +210,14 @@ int search_init(bool replacing, bool use_answer)
 	return -1;
     } else {
 	switch (i) {
-	    case -2:		/* It's the same string. */
+	    case -2:		/* It's an empty string. */
+	    case 0:		/* It's a new string. */
 #ifdef HAVE_REGEX_H
-		/* Since answer is "", use last_search! */
-		if (ISSET(USE_REGEXP) && regexp_init(last_search) == 0)
-		    return -1;
-#endif
-		break;
-	    case 0:		/* They entered something new. */
-		last_replace[0] = '\0';
-#ifdef HAVE_REGEX_H
-		if (ISSET(USE_REGEXP) && regexp_init(answer) == 0)
+		/* Use last_search if answer is an empty string, or
+		 * answer if it isn't. */
+		if (ISSET(USE_REGEXP) &&
+			regexp_init((i == -2) ? last_search :
+			answer) == 0)
 		    return -1;
 #endif
 		break;
@@ -249,8 +241,9 @@ int search_init(bool replacing, bool use_answer)
 		backupstring = mallocstrcpy(backupstring, answer);
 		return -2;	/* Call the opposite search function. */
 	    case NANO_TOGOTOLINE_KEY:
-		do_gotolinecolumn(current->lineno, placewewant + 1,
-			TRUE, TRUE, FALSE);
+		do_gotolinecolumn(openfile->current->lineno,
+			openfile->placewewant + 1, TRUE, TRUE, FALSE,
+			TRUE);
 				/* Put answer up on the statusbar and
 				 * fall through. */
 	    default:
@@ -261,48 +254,24 @@ int search_init(bool replacing, bool use_answer)
     return 0;
 }
 
-bool is_whole_word(size_t pos, const char *buf, const char *word)
-{
-    char *p = charalloc(mb_cur_max()), *r = charalloc(mb_cur_max());
-    size_t word_end = pos + strlen(word);
-    bool retval;
-
-    assert(buf != NULL && pos <= strlen(buf) && word != NULL);
-
-    parse_mbchar(buf + move_mbleft(buf, pos), p, NULL, NULL);
-    parse_mbchar(buf + word_end, r, NULL, NULL);
-
-    /* If we're at the beginning of the line or the character before the
-     * word isn't a non-punctuation "word" character, and if we're at
-     * the end of the line or the character after the word isn't a
-     * non-punctuation "word" character, we have a whole word. */
-    retval = (pos == 0 || !is_word_mbchar(p, FALSE)) &&
-	(word_end == strlen(buf) || !is_word_mbchar(r, FALSE));
-
-    free(p);
-    free(r);
-
-    return retval;
-}
-
 /* Look for needle, starting at (current, current_x).  If no_sameline is
  * TRUE, skip over begin when looking for needle.  begin is the line
- * where we first started searching, at column beginx.  If
+ * where we first started searching, at column begin_x.  If
  * can_display_wrap is TRUE, we put messages on the statusbar, wrap
  * around the file boundaries.  The return value specifies whether we
  * found anything.  If we did, set needle_len to the length of the
  * string we found if it isn't NULL. */
 bool findnextstr(bool can_display_wrap, bool wholeword, bool
-	no_sameline, const filestruct *begin, size_t beginx, const char
+	no_sameline, const filestruct *begin, size_t begin_x, const char
 	*needle, size_t *needle_len)
 {
-    filestruct *fileptr = current;
+    filestruct *fileptr = openfile->current;
     const char *rev_start = NULL, *found = NULL;
     size_t found_len;
 	/* The length of the match we found. */
     size_t current_x_find = 0;
 	/* The location of the match we found. */
-    ssize_t current_y_find = current_y;
+    ssize_t current_y_find = openfile->current_y;
 
     /* rev_start might end up 1 character before the start or after the
      * end of the line.  This won't be a problem because strstrwrapper()
@@ -311,9 +280,10 @@ bool findnextstr(bool can_display_wrap, bool wholeword, bool
      * previous or next line. */
     rev_start =
 #ifndef NANO_SMALL
-	ISSET(BACKWARDS_SEARCH) ? fileptr->data + (current_x - 1) :
+	ISSET(BACKWARDS_SEARCH) ?
+	fileptr->data + (openfile->current_x - 1) :
 #endif
-	fileptr->data + (current_x + 1);
+	fileptr->data + (openfile->current_x + 1);
 
     /* Look for needle in searchstr. */
     while (TRUE) {
@@ -348,7 +318,7 @@ bool findnextstr(bool can_display_wrap, bool wholeword, bool
 	     * a match on the same line we started on and this potential
 	     * match is on that line, continue searching. */
 	    if ((!wholeword || found_whole) && (!no_sameline ||
-		fileptr != current))
+		fileptr != openfile->current))
 		break;
 	}
 
@@ -378,11 +348,11 @@ bool findnextstr(bool can_display_wrap, bool wholeword, bool
 
 #ifndef NANO_SMALL
 	    if (ISSET(BACKWARDS_SEARCH)) {
-		fileptr = filebot;
+		fileptr = openfile->filebot;
 		current_y_find = editwinrows - 1;
 	    } else {
 #endif
-		fileptr = fileage;
+		fileptr = openfile->fileage;
 		current_y_find = 0;
 #ifndef NANO_SMALL
 	    }
@@ -409,10 +379,10 @@ bool findnextstr(bool can_display_wrap, bool wholeword, bool
     /* Ensure we haven't wrapped around again! */
     if (search_last_line &&
 #ifndef NANO_SMALL
-	((!ISSET(BACKWARDS_SEARCH) && current_x_find > beginx) ||
-	(ISSET(BACKWARDS_SEARCH) && current_x_find < beginx))
+	((!ISSET(BACKWARDS_SEARCH) && current_x_find > begin_x) ||
+	(ISSET(BACKWARDS_SEARCH) && current_x_find < begin_x))
 #else
-	current_x_find > beginx
+	current_x_find > begin_x
 #endif
 	) {
 	if (can_display_wrap)
@@ -420,11 +390,11 @@ bool findnextstr(bool can_display_wrap, bool wholeword, bool
 	return FALSE;
     }
 
-    /* Set globals now that we are sure we found something. */
-    current = fileptr;
-    current_x = current_x_find;
-    current_y = current_y_find;
-    placewewant = xplustabs();
+    /* We've definitely found something. */
+    openfile->current = fileptr;
+    openfile->current_x = current_x_find;
+    openfile->current_y = current_y_find;
+    openfile->placewewant = xplustabs();
 
     /* needle_len holds the length of needle. */
     if (needle_len != NULL)
@@ -441,10 +411,11 @@ void findnextstr_wrap_reset(void)
 /* Search for a string. */
 void do_search(void)
 {
-    size_t old_pww = placewewant, fileptr_x = current_x;
+    filestruct *fileptr = openfile->current;
+    size_t fileptr_x = openfile->current_x;
+    size_t old_pww = openfile->placewewant;
     int i;
     bool didfind;
-    filestruct *fileptr = current;
 
 #ifndef DISABLE_WRAPPING
     wrap_reset();
@@ -479,12 +450,13 @@ void do_search(void)
 #endif
 
     findnextstr_wrap_reset();
-    didfind = findnextstr(TRUE, FALSE, FALSE, current, current_x,
-	answer, NULL);
+    didfind = findnextstr(TRUE, FALSE, FALSE, openfile->current,
+	openfile->current_x, answer, NULL);
 
     /* Check to see if there's only one occurrence of the string and
      * we're on it now. */
-    if (fileptr == current && fileptr_x == current_x && didfind) {
+    if (fileptr == openfile->current && fileptr_x ==
+	openfile->current_x && didfind) {
 #ifdef HAVE_REGEX_H
 	/* Do the search again, skipping over the current line, if we're
 	 * doing a bol and/or eol regex search ("^", "$", or "^$"), so
@@ -493,9 +465,10 @@ void do_search(void)
 	 * which case it's the only occurrence. */
 	if (ISSET(USE_REGEXP) && regexp_bol_or_eol(&search_regexp,
 		last_search)) {
-	    didfind = findnextstr(TRUE, FALSE, TRUE, current, current_x,
-		answer, NULL);
-	    if (fileptr == current && fileptr_x == current_x && !didfind)
+	    didfind = findnextstr(TRUE, FALSE, TRUE, openfile->current,
+		openfile->current_x, answer, NULL);
+	    if (fileptr == openfile->current && fileptr_x ==
+		openfile->current_x && !didfind)
 		statusbar(_("This is the only occurrence"));
 	} else {
 #endif
@@ -505,7 +478,7 @@ void do_search(void)
 #endif
     }
 
-    placewewant = xplustabs();
+    openfile->placewewant = xplustabs();
     edit_redraw(fileptr, old_pww);
     search_abort();
 }
@@ -514,9 +487,10 @@ void do_search(void)
 /* Search for the next string without prompting. */
 void do_research(void)
 {
-    size_t old_pww = placewewant, fileptr_x = current_x;
+    filestruct *fileptr = openfile->current;
+    size_t fileptr_x = openfile->current_x;
+    size_t old_pww = openfile->placewewant;
     bool didfind;
-    filestruct *fileptr = current;
 
 #ifndef DISABLE_WRAPPING
     wrap_reset();
@@ -533,12 +507,13 @@ void do_research(void)
 #endif
 
 	findnextstr_wrap_reset();
-	didfind = findnextstr(TRUE, FALSE, FALSE, current, current_x,
-		last_search, NULL);
+	didfind = findnextstr(TRUE, FALSE, FALSE, openfile->current,
+		openfile->current_x, last_search, NULL);
 
 	/* Check to see if there's only one occurrence of the string and
 	 * we're on it now. */
-	if (fileptr == current && fileptr_x == current_x && didfind) {
+	if (fileptr == openfile->current && fileptr_x ==
+		openfile->current_x && didfind) {
 #ifdef HAVE_REGEX_H
 	    /* Do the search again, skipping over the current line, if
 	     * we're doing a bol and/or eol regex search ("^", "$", or
@@ -547,10 +522,11 @@ void do_research(void)
 	     * found again, in which case it's the only occurrence. */
 	    if (ISSET(USE_REGEXP) && regexp_bol_or_eol(&search_regexp,
 		last_search)) {
-		didfind = findnextstr(TRUE, FALSE, TRUE, current,
-			current_x, answer, NULL);
-		if (fileptr == current && fileptr_x == current_x &&
-			!didfind)
+		didfind = findnextstr(TRUE, FALSE, TRUE,
+			openfile->current, openfile->current_x, answer,
+			NULL);
+		if (fileptr == openfile->current && fileptr_x ==
+			openfile->current_x && !didfind)
 		    statusbar(_("This is the only occurrence"));
 	    } else {
 #endif
@@ -562,7 +538,7 @@ void do_research(void)
     } else
         statusbar(_("No current search pattern"));
 
-    placewewant = xplustabs();
+    openfile->placewewant = xplustabs();
     edit_redraw(fileptr, old_pww);
     search_abort();
 }
@@ -570,11 +546,8 @@ void do_research(void)
 
 void replace_abort(void)
 {
-    /* Identical to search_abort(), so we'll call it here.  If it does
-     * something different later, we can change it back.  For now, it's
-     * just a waste to duplicate code. */
+    /* For now, we do the same thing as search_abort(). */
     search_abort();
-    placewewant = xplustabs();
 }
 
 #ifdef HAVE_REGEX_H
@@ -585,18 +558,18 @@ int replace_regexp(char *string, bool create)
      * subexpressions \1 to \9 in the replaced text). */
 
     const char *c = last_replace;
-    size_t search_match_count =
-	regmatches[0].rm_eo - regmatches[0].rm_so;
-    size_t new_line_size =
-	strlen(current->data) + 1 - search_match_count;
+    size_t search_match_count = regmatches[0].rm_eo -
+	regmatches[0].rm_so;
+    size_t new_line_size = strlen(openfile->current->data) + 1 -
+	search_match_count;
 
     /* Iterate through the replacement text to handle subexpression
      * replacement using \1, \2, \3, etc. */
     while (*c != '\0') {
 	int num = (int)(*(c + 1) - '0');
 
-	if (*c != '\\' || num < 1 || num > 9 ||
-		num > search_regexp.re_nsub) {
+	if (*c != '\\' || num < 1 || num > 9 || num >
+		search_regexp.re_nsub) {
 	    if (create)
 		*string++ = *c;
 	    c++;
@@ -613,8 +586,8 @@ int replace_regexp(char *string, bool create)
 	    /* And if create is TRUE, append the result of the
 	     * subexpression match to the new line. */
 	    if (create) {
-		strncpy(string, current->data + current_x +
-			regmatches[num].rm_so, i);
+		strncpy(string, openfile->current->data +
+			openfile->current_x + regmatches[num].rm_so, i);
 		string += i;
 	    }
 	}
@@ -640,8 +613,8 @@ char *replace_line(const char *needle)
     } else {
 #endif
 	search_match_count = strlen(needle);
-	new_line_size = strlen(current->data) - search_match_count +
-	    strlen(answer) + 1;
+	new_line_size = strlen(openfile->current->data) -
+		search_match_count + strlen(answer) + 1;
 #ifdef HAVE_REGEX_H
     }
 #endif
@@ -650,20 +623,21 @@ char *replace_line(const char *needle)
     copy = charalloc(new_line_size);
 
     /* The head of the original line. */
-    strncpy(copy, current->data, current_x);
+    strncpy(copy, openfile->current->data, openfile->current_x);
 
     /* The replacement text. */
 #ifdef HAVE_REGEX_H
     if (ISSET(USE_REGEXP))
-	replace_regexp(copy + current_x, TRUE);
+	replace_regexp(copy + openfile->current_x, TRUE);
     else
 #endif
-	strcpy(copy + current_x, answer);
+	strcpy(copy + openfile->current_x, answer);
 
     /* The tail of the original line. */
-    assert(current_x + search_match_count <= strlen(current->data));
+    assert(openfile->current_x + search_match_count <= strlen(openfile->current->data));
 
-    strcat(copy, current->data + current_x + search_match_count);
+    strcat(copy, openfile->current->data + openfile->current_x +
+	search_match_count);
 
     return copy;
 }
@@ -688,11 +662,11 @@ ssize_t do_replace_loop(const char *needle, const filestruct
     bool begin_line = FALSE, bol_or_eol = FALSE;
 #endif
 #ifndef NANO_SMALL
-    bool old_mark_set = ISSET(MARK_ISSET);
-    filestruct *edittop_save = edittop, *top, *bot;
+    bool old_mark_set = openfile->mark_set;
+    filestruct *edittop_save = openfile->edittop, *top, *bot;
     size_t top_x, bot_x;
     bool right_side_up = FALSE;
-	/* TRUE if (mark_beginbuf, mark_beginx) is the top of the mark,
+	/* TRUE if (mark_begin, mark_begin_x) is the top of the mark,
 	 * FALSE if (current, current_x) is. */
 
     if (old_mark_set) {
@@ -702,8 +676,8 @@ ssize_t do_replace_loop(const char *needle, const filestruct
 	mark_order((const filestruct **)&top, &top_x,
 	    (const filestruct **)&bot, &bot_x, &right_side_up);
 	filepart = partition_filestruct(top, top_x, bot, bot_x);
-	edittop = fileage;
-	UNSET(MARK_ISSET);
+	openfile->edittop = openfile->fileage;
+	openfile->mark_set = FALSE;
 	edit_refresh();
     }
 #endif
@@ -730,13 +704,14 @@ ssize_t do_replace_loop(const char *needle, const filestruct
 	 * beginning line already, and we're still on the beginning line
 	 * after the search, it means that we've wrapped around, so
 	 * we're done. */
-	if (bol_or_eol && begin_line && current == real_current)
+	if (bol_or_eol && begin_line && openfile->current ==
+		real_current)
 	    break;
 	/* Otherwise, set the begin_line flag if we've found a match on
 	 * the beginning line, reset the bol_or_eol flag, and
 	 * continue. */
 	else {
-	    if (current == real_current)
+	    if (openfile->current == real_current)
 		begin_line = TRUE;
 	    bol_or_eol = FALSE;
 	}
@@ -751,9 +726,9 @@ ssize_t do_replace_loop(const char *needle, const filestruct
 
 	if (!replaceall) {
 	    size_t xpt = xplustabs();
-	    char *exp_word = display_string(current->data, xpt,
-		strnlenpt(current->data, current_x + match_len) - xpt,
-		FALSE);
+	    char *exp_word = display_string(openfile->current->data,
+		xpt, strnlenpt(openfile->current->data,
+		openfile->current_x + match_len) - xpt, FALSE);
 
 	    curs_set(0);
 
@@ -791,19 +766,21 @@ ssize_t do_replace_loop(const char *needle, const filestruct
 
 	    copy = replace_line(needle);
 
-	    length_change = strlen(copy) - strlen(current->data);
+	    length_change = strlen(copy) -
+		strlen(openfile->current->data);
 
 #ifndef NANO_SMALL
-	    /* If the mark was on and (mark_beginbuf, mark_begin_x) was
-	     * the top of it, don't change mark_beginx. */
+	    /* If the mark was on and (mark_begin, mark_begin_x) was the
+	     * top of it, don't change mark_begin_x. */
 	    if (!old_mark_set || !right_side_up) {
-		/* Keep mark_beginx in sync with the text changes. */
-		if (current == mark_beginbuf &&
-			mark_beginx > current_x) {
-		    if (mark_beginx < current_x + match_len)
-			mark_beginx = current_x;
+		/* Keep mark_begin_x in sync with the text changes. */
+		if (openfile->current == openfile->mark_begin &&
+			openfile->mark_begin_x > openfile->current_x) {
+		    if (openfile->mark_begin_x < openfile->current_x +
+			match_len)
+			openfile->mark_begin_x = openfile->current_x;
 		    else
-			mark_beginx += length_change;
+			openfile->mark_begin_x += length_change;
 		}
 	    }
 
@@ -812,10 +789,12 @@ ssize_t do_replace_loop(const char *needle, const filestruct
 	    if (!old_mark_set || right_side_up) {
 #endif
 		/* Keep real_current_x in sync with the text changes. */
-		if (current == real_current &&
-			current_x <= *real_current_x) {
-		    if (*real_current_x < current_x + match_len)
-			*real_current_x = current_x + match_len;
+		if (openfile->current == real_current &&
+			openfile->current_x <= *real_current_x) {
+		    if (*real_current_x <
+			openfile->current_x + match_len)
+			*real_current_x = openfile->current_x +
+				match_len;
 		    *real_current_x += length_change;
 		}
 #ifndef NANO_SMALL
@@ -829,20 +808,23 @@ ssize_t do_replace_loop(const char *needle, const filestruct
 #ifndef NANO_SMALL
 	    if (!ISSET(BACKWARDS_SEARCH))
 #endif
-		current_x += match_len + length_change - 1;
+		openfile->current_x += match_len + length_change - 1;
 
 	    /* Cleanup. */
-	    totsize += length_change;
-	    free(current->data);
-	    current->data = copy;
+	    openfile->totsize += length_change;
+	    free(openfile->current->data);
+	    openfile->current->data = copy;
 
 	    if (!replaceall) {
 #ifdef ENABLE_COLOR
-		if (!ISSET(NO_COLOR_SYNTAX))
+		/* If color syntaxes are available and turned on, we
+		 * need to call edit_refresh(). */
+		if (openfile->colorstrings != NULL &&
+			!ISSET(NO_COLOR_SYNTAX))
 		    edit_refresh();
 		else
 #endif
-		    update_line(current, current_x);
+		    update_line(openfile->current, openfile->current_x);
 	    }
 
 	    set_modified();
@@ -856,14 +838,14 @@ ssize_t do_replace_loop(const char *needle, const filestruct
 	 * contains all the text again, set edittop back to what it was
 	 * before, turn the mark back on, and refresh the screen. */
 	unpartition_filestruct(&filepart);
-	edittop = edittop_save;
-	SET(MARK_ISSET);
+	openfile->edittop = edittop_save;
+	openfile->mark_set = TRUE;
 	edit_refresh();
     }
 #endif
 
     /* If text has been added to the magicline, make a new magicline. */
-    if (filebot->data[0] != '\0')
+    if (openfile->filebot->data[0] != '\0')
 	new_magicline();
 
     return numreplaced;
@@ -872,10 +854,10 @@ ssize_t do_replace_loop(const char *needle, const filestruct
 /* Replace a string. */
 void do_replace(void)
 {
-    int i;
     filestruct *edittop_save, *begin;
-    size_t beginx, pww_save;
+    size_t begin_x, pww_save;
     ssize_t numreplaced;
+    int i;
 
     if (ISSET(VIEW_MODE)) {
 	print_view_warning();
@@ -907,6 +889,8 @@ void do_replace(void)
 	last_search = mallocstrcpy(last_search, answer);
     }
 
+    last_replace = mallocstrcpy(last_replace, "");
+
     i = statusq(FALSE, replace_list_2, last_replace,
 #ifndef NANO_SMALL
 	&replace_history,
@@ -933,21 +917,20 @@ void do_replace(void)
     last_replace = mallocstrcpy(last_replace, answer);
 
     /* Save where we are. */
-    edittop_save = edittop;
-    begin = current;
-    beginx = current_x;
-    pww_save = placewewant;
+    edittop_save = openfile->edittop;
+    begin = openfile->current;
+    begin_x = openfile->current_x;
+    pww_save = openfile->placewewant;
 
-    numreplaced = do_replace_loop(last_search, begin, &beginx, FALSE,
+    numreplaced = do_replace_loop(last_search, begin, &begin_x, FALSE,
 	NULL);
 
     /* Restore where we were. */
-    edittop = edittop_save;
-    current = begin;
-    current_x = beginx;
-    placewewant = pww_save;
+    openfile->edittop = edittop_save;
+    openfile->current = begin;
+    openfile->current_x = begin_x;
+    openfile->placewewant = pww_save;
 
-    renumber_all();
     edit_refresh();
 
     if (numreplaced >= 0)
@@ -960,12 +943,15 @@ void do_replace(void)
 
 /* Go to the specified line and column, or ask for them if interactive
  * is TRUE.  Save the x-coordinate and y-coordinate if save_pos is TRUE.
- * Note that both the line and column numbers should be one-based. */
+ * Update the screen afterwards if allow_update is TRUE.  Note that both
+ * the line and column numbers should be one-based. */
 void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
-	bool interactive, bool save_pos)
+	bool interactive, bool save_pos, bool allow_update)
 {
-    if (interactive) {		/* Ask for it. */
+    if (interactive) {
 	char *ans = mallocstrcpy(NULL, answer);
+
+	/* Ask for it. */
 	int i = statusq(FALSE, gotoline_list, use_answer ? ans : "",
 #ifndef NANO_SMALL
 		NULL,
@@ -1001,43 +987,49 @@ void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
 	}
     } else {
 	if (line < 1)
-	    line = current->lineno;
+	    line = openfile->current->lineno;
 
 	if (column < 1)
-	    column = placewewant + 1;
+	    column = openfile->placewewant + 1;
     }
 
-    for (current = fileage; current->next != NULL && line > 1; line--)
-	current = current->next;
+    for (openfile->current = openfile->fileage;
+	openfile->current->next != NULL && line > 1; line--)
+	openfile->current = openfile->current->next;
 
-    current_x = actual_x(current->data, column - 1);
-    placewewant = column - 1;
+    openfile->current_x = actual_x(openfile->current->data, column - 1);
+    openfile->placewewant = column - 1;
 
-    /* If save_pos is TRUE, don't change the cursor position when
-     * updating the edit window. */
+    /* Put the top line of the edit window in range of the current line.
+     * If save_pos is TRUE, don't change the cursor position when doing
+     * it. */
     edit_update(save_pos ? NONE : CENTER);
 
+    /* If allow_update is TRUE, update the screen. */
+    if (allow_update)
+	edit_refresh();
+
     display_main_list();
 }
 
 void do_gotolinecolumn_void(void)
 {
-    do_gotolinecolumn(current->lineno, placewewant + 1, FALSE, TRUE,
-	FALSE);
+    do_gotolinecolumn(openfile->current->lineno,
+	openfile->placewewant + 1, FALSE, TRUE, FALSE, TRUE);
 }
 
-#if defined(ENABLE_MULTIBUFFER) || !defined(DISABLE_SPELLER)
+#ifndef DISABLE_SPELLER
 void do_gotopos(ssize_t line, size_t pos_x, ssize_t pos_y, size_t
 	pos_pww)
 {
     /* Since do_gotolinecolumn() resets the x-coordinate but not the
      * y-coordinate, set the coordinates up this way. */
-    current_y = pos_y;
-    do_gotolinecolumn(line, pos_x + 1, FALSE, FALSE, TRUE);
+    openfile->current_y = pos_y;
+    do_gotolinecolumn(line, pos_x + 1, FALSE, FALSE, TRUE, TRUE);
 
     /* Set the rest of the coordinates up. */
-    placewewant = pos_pww;
-    update_line(current, pos_x);
+    openfile->placewewant = pos_pww;
+    update_line(openfile->current, pos_x);
 }
 #endif
 
@@ -1045,77 +1037,88 @@ void do_gotopos(ssize_t line, size_t pos_x, ssize_t pos_y, size_t
 #ifdef HAVE_REGEX_H
 void do_find_bracket(void)
 {
-    const char *pos, *bracket_pat = "([{<>}])";
-    char cursor_ch, wanted_ch, regexp_pat[] = "[  ]";
+    const char *bracket_pat = "()<>[]{}", *pos;
+    char regex_pat[] = "[  ]", ch, wanted_ch;
+    filestruct *current_save;
     size_t current_x_save, pww_save;
-    int count = 1;
     bool regexp_set = ISSET(USE_REGEXP);
     bool backwards_search_set = ISSET(BACKWARDS_SEARCH);
-    filestruct *current_save;
+    int count = 1;
 
-    cursor_ch = current->data[current_x];
-    pos = strchr(bracket_pat, cursor_ch);
+    assert(strlen(bracket_pat) % 2 == 0);
 
-    if (cursor_ch == '\0' || pos == NULL) {
+    ch = openfile->current->data[openfile->current_x];
+
+    if (ch == '\0' || (pos = strchr(bracket_pat, ch)) == NULL) {
 	statusbar(_("Not a bracket"));
 	return;
     }
 
-    assert(strlen(bracket_pat) % 2 == 0);
+    /* Save where we are. */
+    current_save = openfile->current;
+    current_x_save = openfile->current_x;
+    pww_save = openfile->placewewant;
 
-    wanted_ch =
-	bracket_pat[(strlen(bracket_pat) - 1) - (pos - bracket_pat)];
-
-    current_save = current;
-    current_x_save = current_x;
-    pww_save = placewewant;
+    /* Turn regular expression searches on. */
     SET(USE_REGEXP);
 
-    /* Apparent near redundancy with regexp_pat[] here is needed.
-     * "[][]" works, "[[]]" doesn't. */
-    if (pos < bracket_pat + (strlen(bracket_pat) / 2)) {
-	/* On a left bracket. */
-	regexp_pat[1] = wanted_ch;
-	regexp_pat[2] = cursor_ch;
+    /* If we're on an opening bracket, we want to search forwards for a
+     * closing bracket, and if we're on a closing bracket, we want to
+     * search backwards for an opening bracket.
+     *
+     * We replace the spaces in regex_pat with the opening and closing
+     * brackets, and then do a regular expression search using it.  We
+     * have to put the closing bracket first when we do the latter,
+     * since "[[]]" won't work properly, but "[][]" will. */
+    if ((pos - bracket_pat) % 2 == 0) {
+	wanted_ch = *(pos + 1);
+	regex_pat[1] = wanted_ch;
+	regex_pat[2] = ch;
 	UNSET(BACKWARDS_SEARCH);
     } else {
-	/* On a right bracket. */
-	regexp_pat[1] = cursor_ch;
-	regexp_pat[2] = wanted_ch;
+	wanted_ch = *(pos - 1);
+	regex_pat[1] = ch;
+	regex_pat[2] = wanted_ch;
 	SET(BACKWARDS_SEARCH);
     }
 
-    regexp_init(regexp_pat);
+    /* Compile the regular expression in regex_pat. */
+    regexp_init(regex_pat);
 
-    /* We constructed regexp_pat to be a valid regular expression. */
+    /* The regular expression in regex_pat should always be valid. */
     assert(regexp_compiled);
 
     findnextstr_wrap_reset();
+
     while (TRUE) {
-	if (findnextstr(FALSE, FALSE, FALSE, current, current_x,
-		regexp_pat, NULL)) {
-	    /* Found identical bracket. */
-	    if (current->data[current_x] == cursor_ch)
-		count++;
-	    /* Found complementary bracket. */
-	    else if (--count == 0) {
-		placewewant = xplustabs();
+	if (findnextstr(FALSE, FALSE, FALSE, openfile->current,
+		openfile->current_x, regex_pat, NULL)) {
+	    /* If we found an identical bracket, increment count.  If we
+	     * found a complementary bracket, decrement it. */
+	    count += (openfile->current->data[openfile->current_x] ==
+		ch) ? 1 : -1;
+
+	    /* If count is zero, we've found a matching bracket.  Update
+	     * the screen and get out. */
+	    if (count == 0) {
 		edit_redraw(current_save, pww_save);
 		break;
 	    }
 	} else {
-	    /* Didn't find either a left or right bracket. */
+	    /* We didn't find either an opening or closing bracket.
+	     * Indicate this, restore where we were, and get out. */
 	    statusbar(_("No matching bracket"));
-	    current = current_save;
-	    current_x = current_x_save;
-	    update_line(current, current_x);
+	    openfile->current = current_save;
+	    openfile->current_x = current_x_save;
+	    openfile->placewewant = pww_save;
 	    break;
 	}
     }
 
+    /* Decompile the regular expression in regex_pat. */
     regexp_cleanup();
 
-    /* Restore search direction. */
+    /* Restore search/replace direction. */
     if (backwards_search_set)
 	SET(BACKWARDS_SEARCH);
     else
@@ -1149,17 +1152,26 @@ void history_init(void)
     replacebot = replace_history;
 }
 
+/* Set the current position in the history list h to the bottom. */
+void history_reset(const filestruct *h)
+{
+    if (h == search_history)
+	search_history = searchbot;
+    else if (h == replace_history)
+	replace_history = replacebot;
+}
+
 /* Return the first node containing the first len characters of the
  * string s in the history list, starting at h_start and ending at
  * h_end, or NULL if there isn't one. */
-filestruct *find_history(filestruct *h_start, filestruct *h_end, const
-	char *s, size_t len)
+filestruct *find_history(const filestruct *h_start, const filestruct
+	*h_end, const char *s, size_t len)
 {
-    filestruct *p;
+    const filestruct *p;
 
     for (p = h_start; p != h_end->next && p != NULL; p = p->next) {
 	if (strncmp(s, p->data, len) == 0)
-	    return p;
+	    return (filestruct *)p;
     }
 
     return NULL;
@@ -1199,11 +1211,13 @@ void update_history(filestruct **h, const char *s)
 	bar = p->next;
 	unlink_node(foo);
 	delete_node(foo);
-	renumber(bar);
+	if (bar != NULL)
+	    renumber(bar);
     }
 
     /* If the history is full, delete the beginning entry to make room
-     * for the new entry at the end. */
+     * for the new entry at the end.  We assume that MAX_SEARCH_HISTORY
+     * is greater than zero. */
     if ((*hbot)->lineno == MAX_SEARCH_HISTORY + 1) {
 	filestruct *foo = *hage;
 
@@ -1261,7 +1275,7 @@ char *get_history_newer(filestruct **h)
  * looking at only the first len characters of s, and return that
  * string.  If there isn't one, or if len is 0, don't move h and return
  * s. */
-char *get_history_completion(filestruct **h, char *s, size_t len)
+char *get_history_completion(filestruct **h, const char *s, size_t len)
 {
     assert(s != NULL);
 
@@ -1308,7 +1322,7 @@ char *get_history_completion(filestruct **h, char *s, size_t len)
 
     /* If we're here, we didn't find a match, we didn't find an inexact
      * match, or len is 0.  Return s. */
-    return s;
+    return (char *)s;
 }
-#endif
+#endif /* !DISABLE_TABCOMP */
 #endif /* !NANO_SMALL */
