@@ -1,4 +1,4 @@
-/* $Id: utils.c,v 1.87 2005/06/28 06:25:34 dolorous Exp $ */
+/* $Id: utils.c,v 1.93 2005/07/26 06:13:45 dolorous Exp $ */
 /**************************************************************************
  *   utils.c                                                              *
  *                                                                        *
@@ -24,37 +24,14 @@
 #include <config.h>
 #endif
 
-#include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <pwd.h>
 #include <ctype.h>
 #include <errno.h>
-#include <assert.h>
 #include "proto.h"
 
-#ifdef HAVE_REGEX_H
-#ifdef BROKEN_REGEXEC
-/* Work around a potential segfault in glibc 2.2.3's regexec(). */
-int safe_regexec(const regex_t *preg, const char *string, size_t nmatch,
-	regmatch_t pmatch[], int eflags)
-{
-    if (string != NULL && *string != '\0')
-	return regexec(preg, string, nmatch, pmatch, eflags);
-
-    return REG_NOMATCH;
-}
-#endif
-
-int regexp_bol_or_eol(const regex_t *preg, const char *string)
-{
-    return (regexec(preg, string, 0, NULL, 0) == 0 &&
-	regexec(preg, string, 0, NULL, REG_NOTBOL | REG_NOTEOL) ==
-	REG_NOMATCH);
-}
-#endif /* HAVE_REGEX_H */
-
 int digits(size_t n)
 {
     int i = 1;
@@ -240,6 +217,52 @@ ssize_t ngetdelim(char **lineptr, size_t *n, int delim, FILE *stream)
 #endif
 #endif /* !NANO_SMALL && ENABLE_NANORC */
 
+#ifdef HAVE_REGEX_H
+#ifdef BROKEN_REGEXEC
+/* Work around a potential segfault in glibc 2.2.3's regexec(). */
+int safe_regexec(const regex_t *preg, const char *string, size_t nmatch,
+	regmatch_t pmatch[], int eflags)
+{
+    if (string != NULL && *string != '\0')
+	return regexec(preg, string, nmatch, pmatch, eflags);
+
+    return REG_NOMATCH;
+}
+#endif
+
+int regexp_bol_or_eol(const regex_t *preg, const char *string)
+{
+    return (regexec(preg, string, 0, NULL, 0) == 0 &&
+	regexec(preg, string, 0, NULL, REG_NOTBOL | REG_NOTEOL) ==
+	REG_NOMATCH);
+}
+#endif /* HAVE_REGEX_H */
+
+/* Is the word starting at position pos in buf a whole word? */
+bool is_whole_word(size_t pos, const char *buf, const char *word)
+{
+    char *p = charalloc(mb_cur_max()), *r = charalloc(mb_cur_max());
+    size_t word_end = pos + strlen(word);
+    bool retval;
+
+    assert(buf != NULL && pos <= strlen(buf) && word != NULL);
+
+    parse_mbchar(buf + move_mbleft(buf, pos), p, NULL);
+    parse_mbchar(buf + word_end, r, NULL);
+
+    /* If we're at the beginning of the line or the character before the
+     * word isn't a non-punctuation "word" character, and if we're at
+     * the end of the line or the character after the word isn't a
+     * non-punctuation "word" character, we have a whole word. */
+    retval = (pos == 0 || !is_word_mbchar(p, FALSE)) &&
+	(word_end == strlen(buf) || !is_word_mbchar(r, FALSE));
+
+    free(p);
+    free(r);
+
+    return retval;
+}
+
 /* If we are searching backwards, we will find the last match that
  * starts no later than start.  Otherwise we find the first match
  * starting no earlier than start.  If we are doing a regexp search, we
@@ -375,14 +398,13 @@ char *mallocstrassn(char *dest, char *src)
 /* Append a new magicline to filebot. */
 void new_magicline(void)
 {
-    filebot->next = (filestruct *)nmalloc(sizeof(filestruct));
-    filebot->next->data = mallocstrcpy(NULL, "");
-    filebot->next->prev = filebot;
-    filebot->next->next = NULL;
-    filebot->next->lineno = filebot->lineno + 1;
-    filebot = filebot->next;
-    totlines++;
-    totsize++;
+    openfile->filebot->next = (filestruct *)nmalloc(sizeof(filestruct));
+    openfile->filebot->next->data = mallocstrcpy(NULL, "");
+    openfile->filebot->next->prev = openfile->filebot;
+    openfile->filebot->next->next = NULL;
+    openfile->filebot->next->lineno = openfile->filebot->lineno + 1;
+    openfile->filebot = openfile->filebot->next;
+    openfile->totsize++;
 }
 
 #ifndef NANO_SMALL
@@ -390,85 +412,71 @@ void new_magicline(void)
  * only line in the file. */
 void remove_magicline(void)
 {
-    if (filebot->data[0] == '\0' && filebot->prev != NULL) {
-	filebot = filebot->prev;
-	free_filestruct(filebot->next);
-	filebot->next = NULL;
-	totlines--;
-	totsize--;
+    if (openfile->filebot->data[0] == '\0' &&
+	openfile->filebot->prev != NULL) {
+	openfile->filebot = openfile->filebot->prev;
+	free_filestruct(openfile->filebot->next);
+	openfile->filebot->next = NULL;
+	openfile->totsize--;
     }
 }
 
 /* Set top_x and bot_x to the top and bottom x-coordinates of the mark,
  * respectively, based on the locations of top and bot.  If
  * right_side_up isn't NULL, set it to TRUE If the mark begins with
- * (mark_beginbuf, mark_beginx) and ends with (current, current_x), or
+ * (mark_begin, mark_begin_x) and ends with (current, current_x), or
  * FALSE otherwise. */
 void mark_order(const filestruct **top, size_t *top_x, const filestruct
 	**bot, size_t *bot_x, bool *right_side_up)
 {
     assert(top != NULL && top_x != NULL && bot != NULL && bot_x != NULL);
 
-    if ((current->lineno == mark_beginbuf->lineno && current_x >
-	mark_beginx) || current->lineno > mark_beginbuf->lineno) {
-	*top = mark_beginbuf;
-	*top_x = mark_beginx;
-	*bot = current;
-	*bot_x = current_x;
+    if ((openfile->current->lineno == openfile->mark_begin->lineno &&
+	openfile->current_x > openfile->mark_begin_x) ||
+	openfile->current->lineno > openfile->mark_begin->lineno) {
+	*top = openfile->mark_begin;
+	*top_x = openfile->mark_begin_x;
+	*bot = openfile->current;
+	*bot_x = openfile->current_x;
 	if (right_side_up != NULL)
 	    *right_side_up = TRUE;
     } else {
-	*bot = mark_beginbuf;
-	*bot_x = mark_beginx;
-	*top = current;
-	*top_x = current_x;
+	*bot = openfile->mark_begin;
+	*bot_x = openfile->mark_begin_x;
+	*top = openfile->current;
+	*top_x = openfile->current_x;
 	if (right_side_up != NULL)
 	    *right_side_up = FALSE;
     }
 }
 #endif
 
-/* Calculate the number of lines and the number of characters between
- * begin and end, and return them in lines and size, respectively. */
-void get_totals(const filestruct *begin, const filestruct *end, size_t
-	*lines, size_t *size)
+/* Calculate the number of characters between begin and end, and return
+ * it. */
+size_t get_totsize(const filestruct *begin, const filestruct *end)
 {
+    size_t totsize = 0;
     const filestruct *f;
 
-    if (lines != NULL)
-	*lines = 0;
-    if (size != NULL)
-	*size = 0;
-
     /* Go through the lines from begin to end->prev, if we can. */
-    for (f = begin; f != NULL && f != end; f = f->next) {
-	/* Count this line. */
-	if (lines != NULL)
-	    (*lines)++;
-
+    for (f = begin; f != end && f != NULL; f = f->next) {
 	/* Count the number of characters on this line. */
-	if (size != NULL) {
-	    *size += mbstrlen(f->data);
+	totsize += mbstrlen(f->data);
 
-	    /* Count the newline if we have one. */
-	    if (f->next != NULL)
-		(*size)++;
-	}
+	/* Count the newline if we have one. */
+	if (f->next != NULL)
+	    totsize++;
     }
 
     /* Go through the line at end, if we can. */
     if (f != NULL) {
-	/* Count this line. */
-	if (lines != NULL)
-	    (*lines)++;
-
 	/* Count the number of characters on this line. */
-	if (size != NULL) {
-	    *size += mbstrlen(f->data);
+	totsize += mbstrlen(f->data);
 
-	    /* Count the newline if we have one. */
-	    if (f->next != NULL)
-		(*size)++;
-	}
+	/* Count the newline if we have one. */
+	if (f->next != NULL)
+	    totsize++;
     }
+
+    return totsize;
 }
